/****************************************************************
 *
 * File:	TSBoardPluginMotif.c 
 *
 * Description:	Whiteboard NCS application
 *
 * Author:	Tomasz Stachowiak
 * 
 * Date:	23/07/96
 * 
 * Change Log:
 *
 ****************************************************************/
#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <signal.h>
#include <netdb.h>
#include <netinet/in.h>

#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/Separator.h>
#include <Xm/DrawingA.h>
#include <Xm/MainW.h>
#include <Xm/Label.h>

#include "../lib/common.h"

#include "TSBoardPluginMotif.h"
#include "BoardPluginProtocol.h"
/* #include "Object_procs.h" */

XtAppContext app;            /* global variables */   
XtIntervalId timerId;
XtWorkProc recworkproc;
extern GC gc;
extern Pixmap pixmap;
extern Dimension width, height;
extern Widget drawing_a;

extern struct label_struct param;
extern int current_state;

ncsApplication *ncsapp;

void QueryPluginStatus();
Boolean recWorkProc();

void
QuitApplication(int signo)
{
   CleanupPlugin();
   obj_removeall();
   ncsAppDettach(ncsapp);
   exit(0);
}

main(int argc, char **argv)
{
  Widget toplevel,
            main_w,
               menubar,
               main_panel,
                  buttons_panel,
                     text_but, 
                     line_but, 
                     rec_but,
                     elipse_but,
                     fillrec_but,
                     fillelipse_but,
                     gumka_but,
                  separator,
                  drawcol_panel,
                    /* drawing_a */
                    colors_panel,
                       cb,
                       wb;

  Pixmap colmap;
  XGCValues gcv;
  XmString label, str1, str2, str3, str4;
  XtActionsRec actions;
  String translations = 
     "<Btn1Down>:      draw(down)   ManagerGadgetArm()  \n\
      <Btn1Up>:        draw(up)     ManagerGadgetActivate() \n\
      <Btn1Motion>:     draw(motion) ManagerGadgetButtonMotion()";

  String colors[] = 
    {
       "Black", "Grey", "Red", "Magenta", "Yellow", "Green", "Navy", "Blue", 
       "Purple", "Cyan", "White" 
    };
  int i;

/* Callbacks functions */
  void text_pushed();
  void line_pushed();
  void rec_pushed();
  void fillrec_pushed();
  void elipse_pushed();
  void fillelipse_pushed();
  void gumka_pushed();
  void draw();
  void redraw();
  void file_menu();
  void edit_menu();
  void set_color();

  current_state = LINE_STATE;

/* Top level widget */
  toplevel = XtVaAppInitialize(&app, "Board plugin", NULL, 0, 
			       &argc, argv, NULL,	
			       XmNwidth,        650,
			       XmNheight,       800,
			       NULL);

/* Add new actions */
  actions.string = "draw";
  actions.proc = draw;
  XtAppAddActions(app, &actions, 1);


/* Main window */  
  main_w =  XtVaCreateManagedWidget("main_w", xmMainWindowWidgetClass, 
				    toplevel,
				    XmNscrollingPolicy, XmAUTOMATIC, 
				    XmNvisualPolicy, XmCONSTANT,
				    NULL);

/* Menu bar */
  str1 = XmStringCreateSimple("File");
  str2 = XmStringCreateSimple("Edit");
  menubar = XmVaCreateSimpleMenuBar(main_w, "menubar",
	    XmVaCASCADEBUTTON, str1, 'F',
	    XmVaCASCADEBUTTON, str2, 'E',
	    NULL);

  XmStringFree(str1);
  XmStringFree(str2);
				    
/* File menu, callback: file_menu() */
  str1 = XmStringCreateSimple("Exit");
  str2 = XmStringCreateSimple("Ctrl+C");
  str3 = XmStringCreateSimple("Open");
  str4 = XmStringCreateSimple("Save");
  XmVaCreateSimplePulldownMenu(menubar, "file_menu", 0, file_menu,
	    XmVaPUSHBUTTON, str3, 'O', NULL, NULL,
	    XmVaPUSHBUTTON, str4, 'S', NULL, NULL,
	    XmVaPUSHBUTTON, str1, 'E', "Ctrl<Key>c", str2,
	    NULL);
  XmStringFree(str1);
  XmStringFree(str2);
  XmStringFree(str3);
  XmStringFree(str4);


/* Edit menu, callback: edit_menu()*/
  str1 = XmStringCreateSimple("Clear");
  XmVaCreateSimplePulldownMenu(menubar, "edit_menu", 1, edit_menu,
	    XmVaPUSHBUTTON, str1, 'C', NULL, NULL,
	    NULL);
  XmStringFree(str1);


  XtManageChild(menubar);
  

/* Main panel */
  main_panel =  XtVaCreateWidget("main_panel", xmRowColumnWidgetClass, 
				      main_w,
				      XmNorientation, XmVERTICAL,
				      NULL);

/* Buttons panel */
  buttons_panel =  XtVaCreateWidget("buttons_panel", xmRowColumnWidgetClass, 
				      main_panel,
				      XmNorientation, XmHORIZONTAL,
				      NULL);
  
/* graphic context(GC) creation */
  gcv.foreground = WhitePixelOfScreen(XtScreen(main_w));
  gc = XCreateGC(XtDisplay(main_w),
      RootWindowOfScreen(XtScreen(main_w)), GCForeground, &gcv);

  set_width(buttons_panel, 2);

/* Line button */
  colmap = XCreatePixmap(XtDisplay(buttons_panel), 
			 RootWindowOfScreen(XtScreen(buttons_panel)),
			 16, 16, 
			 DefaultDepthOfScreen(XtScreen(buttons_panel)));
  set_color(buttons_panel, "White");
  XFillRectangle(XtDisplay(main_w), colmap, gc, 0, 0, 16, 16);
  set_color(buttons_panel, "Black");
  XDrawLine(XtDisplay(main_w), colmap, gc, 0, 8, 16, 8);
  line_but = XtVaCreateManagedWidget("line", xmPushButtonWidgetClass, 
				     buttons_panel,
				     XmNlabelType, XmPIXMAP, 
				     XmNlabelPixmap, colmap,
				     NULL);


/* Rectangle button */
  colmap = XCreatePixmap(XtDisplay(buttons_panel), 
			 RootWindowOfScreen(XtScreen(buttons_panel)),
			 16, 16, 
			 DefaultDepthOfScreen(XtScreen(buttons_panel)));
  set_color(buttons_panel, "White");
  XFillRectangle(XtDisplay(main_w), colmap, gc, 0, 0, 16, 16);
  set_color(buttons_panel, "Black");
  XDrawRectangle(XtDisplay(main_w), colmap, gc, 0, 0, 15, 15);
  rec_but = XtVaCreateManagedWidget("rec", xmPushButtonWidgetClass, 
				     buttons_panel,
				     XmNlabelType, XmPIXMAP, 
				     XmNlabelPixmap, colmap,
				     NULL);
/* Filled rectangle button */
  colmap = XCreatePixmap(XtDisplay(buttons_panel), 
			 RootWindowOfScreen(XtScreen(buttons_panel)),
			 16, 16, 
			 DefaultDepthOfScreen(XtScreen(buttons_panel)));
  set_color(buttons_panel, "White");
  XFillRectangle(XtDisplay(main_w), colmap, gc, 0, 0, 16, 16);
  set_color(buttons_panel, "Black");
  XFillRectangle(XtDisplay(main_w), colmap, gc, 0, 0, 16, 16);
  fillrec_but = XtVaCreateManagedWidget("fillrec", xmPushButtonWidgetClass, 
				     buttons_panel,
				     XmNlabelType, XmPIXMAP, 
				     XmNlabelPixmap, colmap,
				     NULL);


/* Elipse button */
  colmap = XCreatePixmap(XtDisplay(buttons_panel), 
			 RootWindowOfScreen(XtScreen(buttons_panel)),
			 16, 16, 
			 DefaultDepthOfScreen(XtScreen(buttons_panel)));
  set_color(buttons_panel, "White");
  XFillRectangle(XtDisplay(main_w), colmap, gc, 0, 0, 16, 16);
  set_color(buttons_panel, "Black");
  XDrawArc(XtDisplay(main_w), colmap, gc, 0, 0, 15, 15, 0, 360*64);
  elipse_but = XtVaCreateManagedWidget("elipse", xmPushButtonWidgetClass, 
				     buttons_panel,
				     XmNlabelType, XmPIXMAP, 
				     XmNlabelPixmap, colmap,
				     NULL);

/* Filled elipse button */
  colmap = XCreatePixmap(XtDisplay(buttons_panel), 
			 RootWindowOfScreen(XtScreen(buttons_panel)),
			 16, 16, 
			 DefaultDepthOfScreen(XtScreen(buttons_panel)));
  set_color(buttons_panel, "White");
  XFillRectangle(XtDisplay(main_w), colmap, gc, 0, 0, 16, 16);
  set_color(buttons_panel, "Black");
  XFillArc(XtDisplay(main_w), colmap, gc, 0, 0, 15, 15, 0, 360*64);
  fillelipse_but = XtVaCreateManagedWidget("fillelipse", xmPushButtonWidgetClass, 
				     buttons_panel,
				     XmNlabelType, XmPIXMAP, 
				     XmNlabelPixmap, colmap,
				     NULL);
/* Text button */
  label = XmStringCreateSimple("T");
  text_but = XtVaCreateManagedWidget("text", xmPushButtonWidgetClass, 
				      buttons_panel,
				      XmNwidth, 100,
				      XmNalignment, XmALIGNMENT_CENTER,
				      XmNlabelString, label,
				      NULL);
  XmStringFree(label);

/* Gumka button */
  label = XmStringCreateSimple("C");
  gumka_but = XtVaCreateManagedWidget("gumka", xmPushButtonWidgetClass, 
				      buttons_panel,
				      XmNwidth, 100,
				      XmNalignment, XmALIGNMENT_CENTER,
				      XmNlabelString, label,
				      NULL);
  XmStringFree(label);

/* Setting callbacks for buttons */
  XtAddCallback(text_but, XmNactivateCallback, text_pushed, NULL);
  XtAddCallback(line_but, XmNactivateCallback, line_pushed, NULL);
  XtAddCallback(rec_but, XmNactivateCallback, rec_pushed, NULL);
  XtAddCallback(elipse_but, XmNactivateCallback, elipse_pushed, NULL);
  XtAddCallback(fillrec_but, XmNactivateCallback, fillrec_pushed, NULL);
  XtAddCallback(fillelipse_but, XmNactivateCallback, fillelipse_pushed, NULL);
  XtAddCallback(gumka_but, XmNactivateCallback, gumka_pushed, NULL);

  XtManageChild(buttons_panel);

/* Separator */
  separator = XmCreateSeparator(main_panel, "separator", NULL, NULL);
  XtManageChild(separator);

  
/* Drawing and colors  panel */
  drawcol_panel =  XtVaCreateWidget("drawcol_panel", xmRowColumnWidgetClass, 
				      main_panel,
				      XmNorientation, XmHORIZONTAL,
				      NULL);

/* Drawing Area widget creation */
  drawing_a = XtVaCreateManagedWidget("drawing_a",
      xmDrawingAreaWidgetClass, drawcol_panel,
      XmNtranslations, XtParseTranslationTable(translations),
      XmNunitType,     Xm1000TH_INCHES,
      XmNwidth,        5000,
      XmNheight,       6000,
      XmNresizePolicy, XmNONE,
      NULL);

  XtAddCallback(drawing_a, XmNexposeCallback, redraw, NULL);

  XtVaSetValues(drawing_a, XmNunitType, XmPIXELS, NULL);
  XtVaGetValues(drawing_a, XmNwidth, &width, XmNheight, &height, NULL);

/* graphic context(GC) creation */
  gcv.foreground = WhitePixelOfScreen(XtScreen(drawing_a));
  gc = XCreateGC(XtDisplay(drawing_a),
      RootWindowOfScreen(XtScreen(drawing_a)), GCForeground, &gcv);

/* Separator */
  separator = XmCreateSeparator(drawcol_panel, "separator1", NULL, NULL);
  XtManageChild(separator);

/* Colors  panel */
  colors_panel =  XtVaCreateWidget("colors_panel", xmRowColumnWidgetClass, 
				      drawcol_panel,
				      XmNorientation, XmVERTICAL,
				      NULL);

/* Width buttons */
  for(i=0; i< 3; i++)
    {
       char lab[20];

       colmap = XCreatePixmap(XtDisplay(colors_panel), 
			      RootWindowOfScreen(XtScreen(colors_panel)),
			      16, 16, 
			      DefaultDepthOfScreen(XtScreen(colors_panel)));
       set_width(colors_panel, 2*i);
       set_color(colors_panel, "White");
       XFillRectangle(XtDisplay(main_w), colmap, gc, 0, 0, 16, 16);
       set_color(colors_panel, "Black");
       XDrawLine(XtDisplay(main_w), colmap, gc, 0, 8, 16, 8);
       sprintf(lab, "wb%d", 2*i);
       wb = XtVaCreateManagedWidget(lab, xmPushButtonWidgetClass, 
				    colors_panel, 
				    XmNlabelType, XmPIXMAP, 
				    XmNlabelPixmap, colmap,
				    NULL);
       XtAddCallback(wb, XmNactivateCallback, set_width, 2*i);
    }

/* Separator */
  separator = XmCreateSeparator(colors_panel, "separator2", NULL, NULL);
  XtManageChild(separator);

 /* Colors buttons */
  for(i=0; i< XtNumber(colors); i++)
    {
       colmap = XCreatePixmap(XtDisplay(colors_panel), 
			      RootWindowOfScreen(XtScreen(colors_panel)),
			      16, 16, 
			      DefaultDepthOfScreen(XtScreen(colors_panel)));
       set_color(colors_panel, colors[i]);
       XFillRectangle(XtDisplay(main_w), colmap, gc, 0, 0, 16, 16);
       cb = XtVaCreateManagedWidget(colors[i], xmPushButtonWidgetClass, 
				    colors_panel, 
				    XmNlabelType, XmPIXMAP, 
				    XmNlabelPixmap, colmap,
				    NULL);
       XtAddCallback(cb, XmNactivateCallback, set_color, colors[i]);
    }


  XtManageChild(colors_panel);
  XtManageChild(drawcol_panel);
  XtManageChild(main_panel);  

  set_color(drawing_a, "Black");

/* Pixel map creation */
  pixmap = XCreatePixmap(XtDisplay(drawing_a),
      RootWindowOfScreen(XtScreen(drawing_a)), width, height,
      DefaultDepthOfScreen(XtScreen(drawing_a)));

/* window clearing */
  XSetForeground(XtDisplay(drawing_a), gc, 
      WhitePixelOfScreen(XtScreen(drawing_a)));

  XFillRectangle(XtDisplay(drawing_a), pixmap, gc, 0, 0, width, height);

  XSetForeground(XtDisplay(drawing_a), gc, 
      BlackPixelOfScreen(XtScreen(drawing_a)));

/* application starting */
  XtRealizeWidget(toplevel);


/* Start up NCS */
  ncsapp = ncsAppAttach(BOARD_PLUGIN_PORT);

  
  recworkproc = XtAppAddWorkProc(app, recWorkProc, NULL);
  
  timerId = XtAppAddTimeOut(app,
			    1000,
			    QueryPluginStatus,
			    NULL); 

  XtAppMainLoop(app);
}


Boolean recWorkProc()
{
  struct timeval timeout;
  fd_set set;
  ncsData *ncsdata;
  int size;
  int sockfd;
  char *data;

  timeout.tv_sec = 0;
  timeout.tv_usec = 100000;

  sockfd = ncsAppGetFD(ncsapp, &set);
  if(select(sockfd+1, &set, 0, 0, &timeout) > 0)
    {
      printf("Processing\n");
      ncsAppProcessIncoming(ncsapp, &set);
      size = ncsAppGetData(ncsapp, &set, &data);
      if(size > 0)
	{
	  printf("Data\n");
	  ncsdata = (ncsData *)data;
	  ProcessPluginMessage(ncsdata->type, ncsdata->data);
	  free(data);
	}

    }

  return False;
}
















