/****************************************************************
 *
 * File:	TSBoardPluginMotif.c 
 *
 * Description:	Whiteboard DVE plugin
 *
 * Author:	Tomasz Stachowiak
 * 
 * Date:	23/07/96
 * 
 * Change Log:
 *
 ****************************************************************/
#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <signal.h>
#include <netdb.h>
#include <netinet/in.h>

#include <dve/dve.h>	    /* DVE includes */
#include <dve/confMgr.h>

#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/Separator.h>
#include <Xm/DrawingA.h>
#include <Xm/MainW.h>
#include <Xm/Label.h>

#include "TSBoardPluginMotif.h"
#include "BoardPluginProtocol.h"


extern XtAppContext app;            /* global variables */   
extern GC gc;
extern Pixmap pixmap;
extern Dimension width, height;
extern Widget drawing_a;

extern struct label_struct param;
extern int current_state;

extern DveSession dveSession;

void
QuitApplication(int signo)
{
   CleanupPlugin();
   exit(0);
}

main(argc, argv)
char *argv[];
{
  Widget toplevel,
            main_w,
               menubar,
               main_panel,
                  buttons_panel,
                     text_but, 
                     line_but, 
                     rec_but,
                  separator
                  /* drawing_a */;

  XGCValues gcv;
  XmString label, str1, str2;
  XtActionsRec actions;
  String translations = 
     "<Btn1Down>:      draw(down)   ManagerGadgetArm()  \n\
      <Btn1Up>:        draw(up)     ManagerGadgetActivate() \n\
      <Btn1Motion>:     draw(motion) ManagerGadgetButtonMotion()";

/* Callbacks functions */
  void text_pushed();
  void line_pushed();
  void rec_pushed();
  void draw();
  void redraw();
  void file_menu();
  void edit_menu();

  current_state = LINE_STATE;

/* Top level widget */
  toplevel = XtVaAppInitialize(&app, "Audio plugin", NULL, 0, 
			       &argc, argv, NULL,	   
			       NULL);
/* Add new actions */
  actions.string = "draw";
  actions.proc = draw;
  XtAppAddActions(app, &actions, 1);

/* Main window */  
  main_w =  XtVaCreateManagedWidget("main_w", xmMainWindowWidgetClass, 
				    toplevel,
				    NULL);

/* Menu bar */
  str1 = XmStringCreateSimple("File");
  str2 = XmStringCreateSimple("Edit");
  menubar = XmVaCreateSimpleMenuBar(main_w, "menubar",
	    XmVaCASCADEBUTTON, str1, 'F',
	    XmVaCASCADEBUTTON, str2, 'E',
	    NULL);

  XmStringFree(str1);
				    
/* File menu, callback: file_menu() */
  str1 = XmStringCreateSimple("Exit");
  str2 = XmStringCreateSimple("Ctrl+C");
  XmVaCreateSimplePulldownMenu(menubar, "file_menu", 0, file_menu,
	    XmVaPUSHBUTTON, str1, 'E', "Ctrl<Key>c", str2,
	    NULL);
  XmStringFree(str1);
  XmStringFree(str2);

/* Edit menu, callback: edit_menu()*/
  str1 = XmStringCreateSimple("Clear");
  XmVaCreateSimplePulldownMenu(menubar, "edit_menu", 1, edit_menu,
	    XmVaPUSHBUTTON, str1, 'C', NULL, NULL,
	    NULL);
  XmStringFree(str1);


  XtManageChild(menubar);
  

/* Main panel */
  main_panel =  XtVaCreateWidget("main_panel", xmRowColumnWidgetClass, 
				      main_w,
				      XmNorientation, XmVERTICAL,
				      NULL);

/* Buttons panel */
  buttons_panel =  XtVaCreateWidget("buttons_panel", xmRowColumnWidgetClass, 
				      main_panel,
				      XmNorientation, XmHORIZONTAL,
				      NULL);
  
/* Line button */
  label = XmStringCreateSimple("LINE");
  line_but = XtVaCreateManagedWidget("line", xmPushButtonWidgetClass, 
				     buttons_panel,
				     XmNlabelString, label,
				     NULL);
  XmStringFree(label);

/* Rectangle button */
  label = XmStringCreateSimple("RECTANGLE");
  rec_but = XtVaCreateManagedWidget("rec", xmPushButtonWidgetClass, 
				     buttons_panel,
				     XmNlabelString, label,
				     NULL);
  XmStringFree(label);

/* Text button */
  label = XmStringCreateSimple("TEXT");
  text_but = XtVaCreateManagedWidget("text", xmPushButtonWidgetClass, 
				      buttons_panel,
				      XmNwidth, 100,
				      XmNalignment, XmALIGNMENT_CENTER,
				      XmNlabelString, label,
				      NULL);
  XmStringFree(label);

/* Setting callbacks for buttons */
  XtAddCallback(text_but, XmNactivateCallback, text_pushed, NULL);
  XtAddCallback(line_but, XmNactivateCallback, line_pushed, NULL);
  XtAddCallback(rec_but, XmNactivateCallback, rec_pushed, NULL);

  XtManageChild(buttons_panel);
  XtManageChild(main_panel);  

/* Separator */
  separator = XmCreateSeparator(main_panel, "separator", NULL, NULL);
  XtManageChild(separator);

/* Drawing Area widget creation */
  drawing_a = XtVaCreateManagedWidget("drawing_a",
      xmDrawingAreaWidgetClass, main_panel,
      XmNtranslations, XtParseTranslationTable(translations),
      XmNunitType,     Xm1000TH_INCHES,
      XmNwidth,        5000,
      XmNheight,       6000,
      XmNresizePolicy, XmNONE,
      NULL);

  XtAddCallback(drawing_a, XmNexposeCallback, redraw, NULL);

  XtVaSetValues(drawing_a, XmNunitType, XmPIXELS, NULL);
  XtVaGetValues(drawing_a, XmNwidth, &width, XmNheight, &height, NULL);

/* Pixel map creation */
  pixmap = XCreatePixmap(XtDisplay(drawing_a),
      RootWindowOfScreen(XtScreen(drawing_a)), width, height,
      DefaultDepthOfScreen(XtScreen(drawing_a)));

/* graphic context(GC) creation */
  gcv.foreground = WhitePixelOfScreen(XtScreen(drawing_a));
  gc = XCreateGC(XtDisplay(drawing_a),
      RootWindowOfScreen(XtScreen(drawing_a)), GCForeground, &gcv);

/* window clearing */
  XSetForeground(XtDisplay(drawing_a), gc, 
      WhitePixelOfScreen(XtScreen(drawing_a)));

  XFillRectangle(XtDisplay(drawing_a), pixmap, gc, 0, 0, width, height);

  XSetForeground(XtDisplay(drawing_a), gc, 
      BlackPixelOfScreen(XtScreen(drawing_a)));

/* application starting */
  XtRealizeWidget(toplevel);


/* Start up OpenDVE */
   if (InitializeOpenDVE(app, AUDIO_PLUGIN_PORT)==NULL)
   {
      fprintf(stderr, "Initialization of OpenDVE Failed...exiting.\n\7");
      exit(0);
   }

  XtAppMainLoop(app);
}


