#include <stdio.h>
#include <stdlib.h>

#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/Separator.h>
#include <Xm/DrawingA.h>
#include <Xm/MainW.h>
#include <Xm/Label.h>

#include "TSBoardPluginMotif.h"
#include "Object_procs.h"

extern XtAppContext app;
GC gc;
Pixmap pixmap;
Dimension width, height;
Widget drawing_a;

struct label_struct param;
extern int current_state;

char current_color[32] = "Black";
int current_width = 0;

void draw_line(Position x, Position y, 
	       Position x1, Position y1, String color, int width);

void draw_gumka(Position x, Position y, 
	       Position x1, Position y1, int tmp);

void clear_line(Position x, Position y, 
	       Position x1, Position y1);

void draw_rectangle(Position x, Position y, 
		    Position x1, Position y1, 
		    String color, int width, int tmp);

void draw_fillrectangle(Position x, Position y, 
			Position x1, Position y1, 
			String color, int width, int tmp);

void clear_rectangle(Position x, Position y, 
		    Position x1, Position y1);

void draw_elipse(Position x, Position y, 
		 Position x1, Position y1, 
		 String color, int width, int tmp);

void draw_fillelipse(Position x, Position y, 
		 Position x1, Position y1, 
		 String color, int width, int tmp);

void clear_elipse(Position x, Position y, 
		    Position x1, Position y1);

void draw_text(Position x, Position y, char *text, String color, int width);

void Clear();

/*********************************************************************
 *
 * Butons callbacks
 *
 *********************************************************************/

void
text_pushed(w, client_data, cbs)
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *cbs;
{
   current_state = TEXT_STATE;
}

void
line_pushed(w, client_data, cbs)
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *cbs;
{
   current_state = LINE_STATE;
}

void
rec_pushed(w, client_data, cbs)
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *cbs;
{
   current_state = RECTANGLE_STATE;
}

void
fillrec_pushed(w, client_data, cbs)
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *cbs;
{
   current_state = FILLRECTANGLE_STATE;
}

void
elipse_pushed(w, client_data, cbs)
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *cbs;
{
   current_state = ELIPSE_STATE;
}

void
fillelipse_pushed(w, client_data, cbs)
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *cbs;
{
   current_state = FILLELIPSE_STATE;
}

void
gumka_pushed(w, client_data, cbs)
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *cbs;
{
   current_state = GUMKA_STATE;
}


/*********************************************************************
 *
 * Colors buttons callback
 *
 *********************************************************************/
void 
set_color(Widget widget, String color)
  {
     Display *dpy = XtDisplay(widget);
     Colormap cmap = DefaultColormapOfScreen(XtScreen(widget));
     XColor col, unused;

     if(!XAllocNamedColor(dpy, cmap, color, &col, &unused))
       {
	  char buf[32];
	  sprintf(buf, "Can't alloc %s", color);
	  XtWarning(buf);
	  return;
       }
     XSetForeground(dpy, gc, col.pixel);
     strcpy(current_color, color);
  }

/*********************************************************************
 *
 * Width buttons callback
 *
 *********************************************************************/
void 
set_width(Widget widget, int width)
  {
     Display *dpy = XtDisplay(widget);

     XSetLineAttributes(dpy, gc, width, LineSolid, CapButt, JoinRound);
     current_width = width;
  }

/*********************************************************************
 *
 * Menus callbacks
 *
 *********************************************************************/

void file_menu(Widget menu_item, int item_no,
		  XmAnyCallbackStruct *cbs)
{
   switch(item_no)
     {
	case 0:
	OpenFile();
	break;
	case 1:
	SaveFile();
	break;
	case 2:
	QuitApplication();
	break;
     }
}

void edit_menu(Widget menu_item,  int item_no,
		  XmAnyCallbackStruct *cbs)
{
   Clear();
   obj_removeall();
   SendClear();
}


/*********************************************************************
 *
 * Drawing area events callback
 *
 *********************************************************************/
void
redraw(w, client_data, cbs)
Widget w;
XtPointer client_data;
XmDrawingAreaCallbackStruct *cbs;
{
   XCopyArea(cbs->event->xexpose.display, pixmap, cbs->window, gc, 0, 0,
	     width, height, 0 ,0);
}


void
draw(w, event, args, num_args)
Widget w;
XButtonEvent *event;
String *args;
int *num_args;
{
   static Position x, y, x1, y1;
   static int draw_state = 1;
   static int up = 1;
   void text_dialog(Widget, int, int);
   GObject *obj;

   if(strcmp(args[0], "up")==0)
      up = 1;
   

   /* Mouse button down */
   if(strcmp(args[0], "down")==0)
     {
	switch(current_state)
	  {
	     case TEXT_STATE:
	     text_dialog(w, event->x, event->y);
	     break;

	     default:
	     x = event->x;
	     y = event->y;
	     x1 = x;
	     y1 = y;
	     draw_state = 2;
	     up = 0;
	     break;
	  }
	up = 0;
     }
	
   /* Mouse motion when button down */
   if((strcmp(args[0], "motion") == 0) && (draw_state == 2) && (up == 0))
     {
	XCopyArea(XtDisplay(drawing_a), pixmap, 
		  XtWindow(drawing_a), gc, 0, 0, width, height, 0 ,0);
	switch(current_state)
	  {
	     case LINE_STATE:
	     XDrawLine(XtDisplay(drawing_a), XtWindow(drawing_a), gc, 
		       x, y, event->x, event->y);
	     break;

	     case RECTANGLE_STATE:
	     draw_rectangle(x, y, event->x, event->y, 
			    current_color, current_width, 1);
	     break;

	     case FILLRECTANGLE_STATE:
	     draw_fillrectangle(x, y, event->x, event->y, 
			    current_color, current_width, 1);
	     break;

	     case ELIPSE_STATE:
	     draw_elipse(x, y, event->x, event->y, 
			 current_color, current_width, 1);
	     break;

	     case FILLELIPSE_STATE:
	     draw_fillelipse(x, y, event->x, event->y, 
			 current_color, current_width, 1);
	     break;

	     case GUMKA_STATE:
	     draw_gumka(x, y, event->x, event->y, 1);
	     break;
	  }
     }

   /* Mouse button up */
   if(strcmp(args[0], "up")==0)
     {
	switch(current_state)
	  {
	     case LINE_STATE:
	     obj = obj_add(LINE, x, y, event->x, event->y, "", 
			   current_color, current_width);
	     obj_draw(obj);
	     obj_send(obj);
	     break;

	     case RECTANGLE_STATE:
	     obj = obj_add(RECTANGLE, x, y, event->x, event->y, "",
		     current_color, current_width);
	     obj_draw(obj);
	     obj_send(obj);
	     break;

	     case FILLRECTANGLE_STATE:
	     obj = obj_add(FILLRECTANGLE, x, y, event->x, event->y, "",
		     current_color, current_width);
	     obj_draw(obj);
	     obj_send(obj);
	     break;

	     case ELIPSE_STATE:
	     obj = obj_add(ELIPSE, x, y, event->x, event->y, "",
		     current_color, current_width);
	     obj_draw(obj);
	     obj_send(obj);
	     break;

	     case FILLELIPSE_STATE:
	     obj = obj_add(FILLELIPSE, x, y, event->x, event->y, "",
		     current_color, current_width);
	     obj_draw(obj);
	     obj_send(obj);
	     break;

	     case GUMKA_STATE:
	     obj = obj_add(GUMKA, x, y, event->x, event->y, "",
		     current_color, current_width);
	     obj_draw(obj);
	     obj_send(obj);
	     break;

	     case TEXT_STATE: break;
	  }
	draw_state = 1;
	up = 1;
     }
}

/*********************************************************************
 *
 * Text dialog
 *
 *********************************************************************/
void
text_dialog(Widget w, int x, int y)
  {
     static Widget dialog;
     XmString t = XmStringCreateSimple("Enter text:");
     extern void read_str();
     Arg args[2];

     param.x = x;
     param.y = y;
     XtSetArg(args[0], XmNselectionLabelString, t);
     XtSetArg(args[1], XmNautoUnmanage, False);
     dialog = XmCreatePromptDialog(w, "prompt", args, 2);
     XmStringFree(t);


     XtAddCallback(dialog, XmNokCallback, read_str, &param);
     XtAddCallback(dialog, XmNcancelCallback, XtDestroyWidget, NULL);

     XtSetSensitive(
	  XmSelectionBoxGetChild(dialog, XmDIALOG_HELP_BUTTON), False);
     XtManageChild(dialog);

     XtPopup(XtParent(dialog), XtGrabNone);

  }

/*********************************************************************
 *
 * Text dialog OK callback
 *
 *********************************************************************/
void
read_str(w, param, cbs)
Widget w;
struct label_struct *param;
XmSelectionBoxCallbackStruct *cbs;
{
   char *text;
   GObject *obj;

   if(XmStringGetLtoR(cbs->value, XmSTRING_DEFAULT_CHARSET, &text))
     {
	obj = obj_add(TEXT, param->x, param->y, 0, 0, text, 
		current_color, current_width);
	obj_draw(obj);
	obj_send(obj);
	XtFree(text);
     }
   XtDestroyWidget(w);
}

/*********************************************************************
 *
 * Display functions
 *
 *********************************************************************/
void draw_line(Position x, Position y, 
	       Position x1, Position y1, String color, int width)
  {
     char pom[32];
     int tmp_width;

     strcpy(pom, current_color);
     tmp_width = current_width;

     set_color(drawing_a, color);
     set_width(drawing_a, width);

     XDrawLine(XtDisplay(drawing_a), XtWindow(drawing_a), gc, 
	       x, y, x1, y1);
     XDrawLine(XtDisplay(drawing_a), pixmap, gc, 
	       x, y, x1, y1);

     strcpy(current_color, pom);
     set_color(drawing_a, current_color);
     current_width = tmp_width;
     set_width(drawing_a, current_width);
  }

void draw_rectangle(Position x, Position y, Position x1, Position y1, 
		    String color, int width, int tmp)
  {
     Position pom;
     char col[32];
     int tmp_width;

     tmp_width = current_width;
     set_width(drawing_a, width);

     strcpy(col, current_color);
	
     set_color(drawing_a, color);
     if (x > x1)
       {
	  pom = x;
	  x = x1;
	  x1 = pom;
       }
     if (y > y1)
       {
	  pom = y;
	  y = y1;
	  y1 = pom;
       }
     
     XDrawRectangle(XtDisplay(drawing_a), XtWindow(drawing_a), gc, 
		    x, y, x1 - x, y1 - y);
     if(!tmp)
	XDrawRectangle(XtDisplay(drawing_a), pixmap, gc, 
		       x, y, x1 - x, y1 - y);

     strcpy(current_color, col);
     set_color(drawing_a, current_color);
     current_width = tmp_width;
     set_width(drawing_a, current_width);
  }

void draw_fillrectangle(Position x, Position y, Position x1, Position y1, 
		    String color, int width, int tmp)
  {
     Position pom;
     char col[32];
     int tmp_width;

     tmp_width = current_width;
     set_width(drawing_a, width);

     strcpy(col, current_color);
	
     set_color(drawing_a, color);
     if (x > x1)
       {
	  pom = x;
	  x = x1;
	  x1 = pom;
       }
     if (y > y1)
       {
	  pom = y;
	  y = y1;
	  y1 = pom;
       }
     
     XFillRectangle(XtDisplay(drawing_a), XtWindow(drawing_a), gc, 
		    x, y, x1 - x, y1 - y);
     if(!tmp)
	XFillRectangle(XtDisplay(drawing_a), pixmap, gc, 
		       x, y, x1 - x, y1 - y);

     strcpy(current_color, col);
     set_color(drawing_a, current_color);
     current_width = tmp_width;
     set_width(drawing_a, current_width);
  }

void draw_elipse(Position x, Position y, Position x1, Position y1, 
		 String color, int width, int tmp)
  {
     Position pom;
     char col[32];
     int tmp_width;

     tmp_width = current_width;
     set_width(drawing_a, width);
     strcpy(col, current_color);
	
     set_color(drawing_a, color);
     if (x > x1)
       {
	  pom = x;
	  x = x1;
	  x1 = pom;
       }
     if (y > y1)
       {
	  pom = y;
	  y = y1;
	  y1 = pom;
       }
     
     XDrawArc(XtDisplay(drawing_a), XtWindow(drawing_a), gc, 
		    x, y, x1 - x, y1 - y, 0, 360*64);
     if(!tmp)
	XDrawArc(XtDisplay(drawing_a), pixmap, gc, 
		       x, y, x1 - x, y1 - y, 0, 360*64);

     strcpy(current_color, col);
     set_color(drawing_a, current_color);
     current_width = tmp_width;
     set_width(drawing_a, current_width);
  }

void draw_fillelipse(Position x, Position y, Position x1, Position y1, 
		 String color, int width, int tmp)
  {
     Position pom;
     char col[32];
     int tmp_width;

     tmp_width = current_width;
     set_width(drawing_a, width);
     strcpy(col, current_color);
	
     set_color(drawing_a, color);
     if (x > x1)
       {
	  pom = x;
	  x = x1;
	  x1 = pom;
       }
     if (y > y1)
       {
	  pom = y;
	  y = y1;
	  y1 = pom;
       }
     
     XFillArc(XtDisplay(drawing_a), XtWindow(drawing_a), gc, 
		    x, y, x1 - x, y1 - y, 0, 360*64);
     if(!tmp)
	XFillArc(XtDisplay(drawing_a), pixmap, gc, 
		       x, y, x1 - x, y1 - y, 0, 360*64);

     strcpy(current_color, col);
     set_color(drawing_a, current_color);
     current_width = tmp_width;
     set_width(drawing_a, current_width);
  }
void draw_gumka(Position x, Position y, Position x1, Position y1, int tmp)
  {
     Position pom;

     if (x > x1)
       {
	  pom = x;
	  x = x1;
	  x1 = pom;
       }
     if (y > y1)
       {
	  pom = y;
	  y = y1;
	  y1 = pom;
       }
     XSetForeground(XtDisplay(drawing_a), gc, 
		    WhitePixelOfScreen(XtScreen(drawing_a)));
     
     if(!tmp)
	XFillRectangle(XtDisplay(drawing_a), pixmap, gc, x, y, x1-x, y1-y);
     XFillRectangle(XtDisplay(drawing_a), XtWindow(drawing_a), gc, 
		    x, y, x1-x, y1-y);
     
     set_color(drawing_a, current_color);
  }


void draw_text(Position x, Position y, char *text,
	       String color, int width)
  {
     char pom[32];
     int tmp_width;

     tmp_width = current_width;
     set_width(drawing_a, width);

     strcpy(pom, current_color);
     set_color(drawing_a, color);

     XDrawString(XtDisplay(drawing_a), XtWindow(drawing_a), gc, 
		 x, y, text, strlen(text));
     XDrawString(XtDisplay(drawing_a), pixmap, gc, 
		 x, y, text, strlen(text));

     strcpy(current_color, pom);
     set_color(drawing_a, current_color);
     current_width = tmp_width;
     set_width(drawing_a, current_width);
  }

/*
 *  Clear function
 */

void Clear()
  {
  XSetForeground(XtDisplay(drawing_a), gc, 
      WhitePixelOfScreen(XtScreen(drawing_a)));

  XFillRectangle(XtDisplay(drawing_a), pixmap, gc, 0, 0, width, height);

  set_color(drawing_a, current_color);
/*  XSetForeground(XtDisplay(drawing_a), gc, 
      BlackPixelOfScreen(XtScreen(drawing_a)));
*/  XCopyArea(XtDisplay(drawing_a), pixmap, XtWindow(drawing_a), gc,
	    0, 0, width, height, 0, 0);

  }


/*
 *  File menu functions
 */

void open_ok();
void save();

OpenFile()
  {
     Widget file_dlg;

     file_dlg = XmCreateFileSelectionDialog(drawing_a, "file_open_dlg",
                NULL, 0);
     XtAddCallback(file_dlg, XmNcancelCallback, XtDestroyWidget, NULL);
     XtAddCallback(file_dlg, XmNokCallback, open_ok, NULL);

     XtManageChild(file_dlg);
  }


void open_ok(Widget w, XtPointer client_data, 
	  XmFileSelectionBoxCallbackStruct *cbs)
  {
     FILE *file;
     char *file_name;

     if (!XmStringGetLtoR(cbs->value, XmSTRING_DEFAULT_CHARSET, &file_name))
	return;
     
     if (!*file_name)
       {
	  printf("No file selected\n");
	  XtFree(file_name);
	  return;
       }

     file = fopen(file_name, "rb");

     Clear();
     obj_removeall();
     obj_restoreall(file);
     obj_drawall();
     obj_sendall();

     fclose(file);

     XtFree(file_name);
     XtDestroyWidget(w);
  }

SaveFile()
  {
     Widget file_dlg;

     file_dlg = XmCreateFileSelectionDialog(drawing_a, "file_save_dlg",
                NULL, 0);
     XtAddCallback(file_dlg, XmNcancelCallback, XtDestroyWidget, NULL);
     XtAddCallback(file_dlg, XmNokCallback, save, NULL);

     XtManageChild(file_dlg);
  }

void save(Widget w, XtPointer client_data, 
	  XmFileSelectionBoxCallbackStruct *cbs)
  {
     FILE *file;
     char *file_name;

     if (!XmStringGetLtoR(cbs->value, XmSTRING_DEFAULT_CHARSET, &file_name))
	return;
     
     if (!*file_name)
       {
	  printf("No file selected\n");
	  XtFree(file_name);
	  return;
       }

     file = fopen(file_name, "wb");
     obj_saveall(file);
     fclose(file);

     XtFree(file_name);
     XtDestroyWidget(w);
  }
