/***************************************************************************
 *
 * File name               :browser.c
 * 
 * Description             :NCS conference manager
 *
 * Authors                 :Tomasz Stachowiak, NPAC
 *
 * Date                    :10-23-96
 *
 ***************************************************************************/
#define XP_UNIX

#include <stdio.h>
#include <stdlib.h>

#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/LabelG.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/Form.h>
#include <Xm/DialogS.h>
#include <X11/Shell.h>
#include <Xm/MessageB.h>
#include <Xm/List.h>

/* Motif resource manager (for .uid interface files)*/
#include <Mrm/MrmPublic.h>

#include "launch.c"
#include "npapi.h"
#include "../lib/common.h"

#define MIME_EXT_DESCRIPTION "application/x-dve:.dve"
/* In case of stupid server "text/plain:.dve" */

typedef struct _PluginInstance
{
    Widget	netscape_widget; /* The widget provided by netscape */
    Display    *display;         /* Netscape's display connection */
    uint16	mode;            /* NP_EMBED, NP_FULL, or NP_BACKGROUND */
    uint32 width, height;
    
    int len;
    XtIntervalId timerId;
    XtAppContext appContext;
} PluginInstance;



XtAppContext    AppContext;
Widget          parent;
Widget          pluginsLabel, membersLabel, pluginsList, membersAccList;
XmStringTable   plugTlist;

/* Add member dialog widgets */
Widget	 mainWin, mainForm, textLine, bMemOK, bMemCancel,
         usr_text, host_text;


Participant invitingPart;
XtWorkProc recworkproc;

char *information;
Arg wArg[10];
int numArg;
XmString label;
int err;


Boolean recWorkProc();

void start_conference();
void exit_conference();
void partyAccepted();
void partyRejected();
void partyLeft(Participant *part);
void partyNew(Participant *part);
void SetMessage(const char *msg);
void createMembersInterface();

int init_plugins();

void parseInformation(Widget list, char *info);
/* 
 * Main panel buttons callbacks 
 */
static void add_member(Widget w, XtPointer data1, XtPointer data2);
static void info_plugin(Widget w, XtPointer data1, XtPointer data2);
static void show_plugin(Widget w, XtPointer data1, XtPointer data2);
static void hide_plugin(Widget w, XtPointer data1, XtPointer data2);
static void accept_invitation(Widget w, XtPointer data1, XtPointer data2);
static void reject_invitation(Widget w, XtPointer data1, XtPointer data2);

/* Add dialog buttons callbacks */
void add_usr(Widget w, XtPointer app_data, XtPointer widget_data);
void delete_usr(Widget w, XtPointer app_data, XtPointer widget_data);
void invite_usrs(Widget w, XtPointer app_data, XtPointer widget_data);
void add_cancel(Widget w, XtPointer app_data, XtPointer widget_data);

/* Mrm create callbacks */
void statuslabel_created(Widget w, XtPointer app_data, XtPointer widget_data);
void listform_created(Widget w, XtPointer app_data, XtPointer widget_data);
void add_but_created(Widget w, XtPointer app_data, XtPointer widget_data);
void accept_but_created(Widget w, XtPointer app_data, XtPointer widget_data);
void reject_but_created(Widget w, XtPointer app_data, XtPointer widget_data);
void show_but_created(Widget w, XtPointer app_data, XtPointer widget_data);
void hide_but_created(Widget w, XtPointer app_data, XtPointer widget_data);
void info_but_created(Widget w, XtPointer app_data, XtPointer widget_data);
void users_list_created(Widget w, XtPointer app_data, XtPointer widget_data);
void activeusr_list_created(Widget w, XtPointer app_data, XtPointer widget_data);
void user_field_created(Widget w, XtPointer app_data, XtPointer widget_data);
void host_field_created(Widget w, XtPointer app_data, XtPointer widget_data);


/* 
 * Mrm variables 
 */
const String  file_names[2] = { "/main.uid", "/add.uid"};     
String exact_file_names[2];
MrmHierarchy   hierarchy_id;     
MrmType        main_window_class, add_panel_class;

/* main panel widgets */
Widget         main_window, listform, statuslabel, add_but, accept_but, 
               reject_but, show_but, hide_but, info_but;

/* add dialog widgets */
Widget         users_list, activeusr_list, add_panel, user_field, host_field;

static MrmRegisterArg cb_list[] = 
{
  { "add_member",            (XtPointer)add_member            },
  { "accept_invitation",     (XtPointer)accept_invitation     },
  { "reject_invitation",     (XtPointer)reject_invitation     },
  { "show_plugin",           (XtPointer)show_plugin           },
  { "hide_plugin",           (XtPointer)hide_plugin           },
  { "info_plugin",           (XtPointer)info_plugin           },
  { "add_usr",               (XtPointer)add_usr               },
  { "delete_usr",            (XtPointer)delete_usr            },
  { "invite_usrs",           (XtPointer)invite_usrs           },
  { "add_cancel",            (XtPointer)add_cancel            },
  { "statuslabel_created",   (XtPointer)statuslabel_created   },
  { "listform_created",      (XtPointer)listform_created      },
  { "add_but_created",       (XtPointer)add_but_created       },
  { "accept_but_created",    (XtPointer)accept_but_created    },
  { "reject_but_created",    (XtPointer)reject_but_created    },
  { "show_but_created",      (XtPointer)show_but_created      },
  { "hide_but_created",      (XtPointer)hide_but_created      },
  { "info_but_created",      (XtPointer)info_but_created      },
  { "users_list_created",    (XtPointer)users_list_created    },
  { "activeusr_list_created",(XtPointer)activeusr_list_created},
  { "user_field_created",    (XtPointer)user_field_created    },
  { "host_field_created",    (XtPointer)host_field_created    }
};


/***********************************************************************
 *
 * Implementations of Netscape's plugin API functions
 *
 ***********************************************************************/

char *
NPP_GetMIMEDescription(void)
{
#ifdef DISP
  printf( "OpenDVE plugin:MIME description loaded\n");  
#endif
  return( MIME_EXT_DESCRIPTION );
}

NPError
NPP_Initialize(void)
{
#ifdef DISP
  printf( "OpenDVE:Plugin Initialized\n");  
#endif
  return NPERR_NO_ERROR;
}

void
NPP_Shutdown(void)
{
#ifdef DISP
  printf( "OpenDVE:Plugin Shutdown called\n");  
#endif
}


NPError 
NPP_New(NPMIMEType pluginType,
	NPP instance,
	uint16 mode,
	int16 argc,
	char* argn[],
	char* argv[],
	NPSavedData* saved)
{
        PluginInstance* This;

	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;
		
	instance->pdata = NPN_MemAlloc(sizeof(PluginInstance));
	
	This = (PluginInstance*) instance->pdata;
#ifdef DISP
	  printf( "OpenDVE: NPP_New called\n");
#endif
	if (This != NULL)
	{
   	        int i;
		This->netscape_widget = NULL;
		This->display = NULL;
		This->mode = NP_FULL;   
		This->width = 400;
		This->height = 500;
#ifdef DISP
	  printf( "OpenDVE: NPP_New called Mode=%d\n", This->mode);  
#endif

		/*--- Start custom code */
		This->len = 0;
		This->timerId = 0;

		/*--- End custom code */
		
		return NPERR_NO_ERROR;
	}
	else
		return NPERR_OUT_OF_MEMORY_ERROR;
}/* NPP_New */


NPError 
NPP_Destroy(NPP instance, NPSavedData** save)
{
	PluginInstance* This;

	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;

	This = (PluginInstance*) instance->pdata;

#ifdef DISP
	printf( "OpenDVE: NPP_Destroy called\n");  
#endif
	exit_conference();

	if (This != NULL) {

		/*--- Start custom code */

		/* Stream is closed, we have the data, so start a timer to
		 * call the draw routine
		 */

		if(This->timerId)
		    XtRemoveTimeOut(This->timerId);
		This->timerId = 0;

		/*--- End custom code */

		NPN_MemFree(instance->pdata);
		instance->pdata = NULL;
	}

	return NPERR_NO_ERROR;
}/* NPP_Destroy */

NPError 
NPP_SetWindow(NPP instance, NPWindow* window)
{
	PluginInstance* This;
	char *env;
	int i;

	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;

	This = (PluginInstance*) instance->pdata;
#ifdef DISP	
	printf( "OpenDVE: NPP_SetWindow called for %d\n", (int)instance);  
#endif

	This->display = ((NPSetWindowCallbackStruct *)window->ws_info)->display;
	This->netscape_widget = XtWindowToWidget(This->display, (Window)window->window);
	This->width = window->width;
	This->height = window->height;
	
	This->appContext=XtWidgetToApplicationContext(This->netscape_widget);
	AppContext = This->appContext;
	parent=This->netscape_widget;
	

	/* Mrm operations */
	MrmInitialize();     
	env = getenv("HOME");
	for(i=0; i<2; i++)
	  {
	    exact_file_names[i] = malloc(100);
	    strcpy(exact_file_names[i], env);
	    strcat(exact_file_names[i], file_names[i]);
	  }

	MrmOpenHierarchyPerDisplay(XtDisplay(parent), 
				   2, 
				   exact_file_names,              
				   (MrmOsOpenParamPtr *) NULL, 
				   &hierarchy_id);

	if (MrmRegisterNames(cb_list, 
			     (MrmCount)XtNumber(cb_list)) != MrmSUCCESS)
	  {
	    fprintf(stderr, "Unable to register callbacks\n");
	    fflush(stderr);
	  }
	
	if (MrmFetchWidget(hierarchy_id, "simpleWindow",
			   parent, &main_window, &main_window_class)
	    != MrmSUCCESS) 
	  {
	    fprintf(stderr, "Unable to fetch top-level widget.\n");
	    fflush(stderr);
	  }      

	XtManageChild(main_window);
	XtRealizeWidget(parent);

	return NPERR_NO_ERROR;

} /* NPP_SetWindow */


NPError 
NPP_NewStream(NPP instance,
	      NPMIMEType type,
	      NPStream *stream, 
	      NPBool seekable,
	      uint16 *stype)
{
	PluginInstance* This;

	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;

	This = (PluginInstance*) instance->pdata;

/*--- Start custom code */
#ifdef DISP
	printf( "OpenDVE: NPP_NewStream called\n");  
#endif

	*stype = NP_NORMAL;
	
	/* localURL = changeURL(stream->url);*/
	
/*--- End custom code */
	return NPERR_NO_ERROR;
}/* NPP_NewStream */


int32 
NPP_WriteReady(NPP instance, NPStream *stream)
{
  
	PluginInstance* This;
	if (instance != NULL)
		This = (PluginInstance*) instance->pdata;
	
#ifdef DISP
	printf( "OpenDVE: NPP_WriteReady returns %d\n", req);  
#endif

	return 1024; 
	
}


int32 
NPP_Write(NPP instance, NPStream *stream, int32 offset, int32 len, void *buffer)
{
	int i;
	PluginInstance* This;
	

	if (instance != NULL)
	{
		This = (PluginInstance*) instance->pdata;
	      }

	information = XtRealloc(information, This->len + len + 2);

	strncpy(&(information[This->len]), buffer, len);

	This->len += len;

	return len;		/* The number of bytes accepted */
}


NPError 
NPP_DestroyStream(NPP instance, NPStream *stream, NPError reason)
{
	PluginInstance* This;
	XCharStruct bounds;
	int direction, ascent, descent;

	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;
	This = (PluginInstance*) instance->pdata;
	

	/*
	 * adding timer for the main initialization procedure 
	 */
	This->timerId = XtAppAddTimeOut(This->appContext,
					100,
					start_conference,
					(XtPointer)instance); 

	return NPERR_NO_ERROR;
}


void 
NPP_StreamAsFile(NPP instance, NPStream *stream, const char* fname)
{
}

void 
NPP_Print(NPP instance, NPPrint* printInfo)
{
}


/*********************************************************************
 *
 * Mrm create callbacks
 *
 *********************************************************************/
void statuslabel_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  statuslabel = w;
  SetMessage("Initializing Conference Engine...");
}

void listform_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  listform = w;
}

void add_but_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  add_but = w;
}

void accept_but_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  accept_but = w;
  XtSetSensitive(w, False);
}

void reject_but_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  reject_but = w;
  XtSetSensitive(w, False);
}

void show_but_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  show_but = w;
  XtSetSensitive(w, False);
}
void hide_but_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  hide_but = w;
  XtSetSensitive(w, False);
}


void info_but_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  info_but = w;
  XtSetSensitive(w, False);
}

void users_list_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  users_list = w;
}

void activeusr_list_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  activeusr_list = w;
}

void user_field_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  user_field = w;
}

void host_field_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  host_field = w;
}



/*************************************************************************
  *
  *  Callbacks invoked by buttons
  *
  ************************************************************************/
showMessage(Widget w, char *msg)
{
  Widget dialog;
  XmString text;
  XmString title;

  text = XmStringCreateLtoR(msg, XmSTRING_DEFAULT_CHARSET);
  title = XmStringCreateSimple("Message");
  dialog = XmCreateMessageDialog(w, "message", NULL, 0);
  XtVaSetValues(dialog,
		XmNmessageString, text,
		XmNdialogType, XmDIALOG_INFORMATION,
		XmNdialogTitle, title,
		NULL);
  XmStringFree(text);
  XmStringFree(title);

  XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON));
  XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_CANCEL_BUTTON));

  XtManageChild(dialog);
  XtPopup(XtParent(dialog), XtGrabNone);
}


/***********************************
 * 
 * Add member dialog
 * 
 **********************************/
void
add_member(Widget w, XtPointer data1, XtPointer data2)
{
  Widget usr_lab, host_lab;

  mainWin = XtVaCreatePopupShell( "mainWin",
				 xmDialogShellWidgetClass,
				 parent,
				 XmNx, 370,
				 XmNy, 300,
				 XmNwidth, 500,
				 XmNheight, 470,
				 XmNtitle, "Add members",
				 NULL );

  

  if (MrmFetchWidget(hierarchy_id, "addWindow",
		     mainWin, &add_panel, &add_panel_class)
      != MrmSUCCESS) 
    {
      fprintf(stderr, "Unable to fetch top-level 'add' widget.\n");
      fflush(stderr);
    }      

  XtManageChild(add_panel);

  XtRealizeWidget(mainWin);

  XtPopup(mainWin, XtGrabNone);

  parseInformation(users_list, information);

  return;
}

/*
 * Add member dialog callbacks 
 */

/* Cancel callback */
void add_cancel(Widget w, XtPointer app_data, XtPointer widget_data)
{
  XtDestroyWidget(mainWin);
}

/* Activate user (add button) */
void add_usr(Widget w, XtPointer app_data, XtPointer widget_data)
{
  int sel_no;
  XmStringTable sel_item;
  char *user;
  char *host;
  char malpa = '@';
  char addr[200];
  XmString address;

  XtVaGetValues(users_list,
		XmNselectedItemCount, &sel_no,
		NULL);
  if(sel_no > 0)
    {
      XtVaGetValues(users_list,
		    XmNselectedItems, &sel_item,
		    NULL);
      if(!XmListItemExists(activeusr_list, sel_item[0]))
	XmListAddItem(activeusr_list, sel_item[0], 0);
    }

  XtVaGetValues(user_field,
		XmNvalue, &user,
		NULL);
  XtVaGetValues(host_field,
		XmNvalue, &host,
		NULL);
  if((strlen(user) > 0) && (strlen(host) > 0))
    {
      strcpy(addr, user);
      strcat(addr, &malpa);
      strcpy(&addr[strlen(user) + 1], host);
      address = XmStringCreateSimple(addr);
      if(!XmListItemExists(activeusr_list, address))
	{
	  XmListAddItem(activeusr_list, address, 0);
	  XmTextFieldSetString(user_field, "");
	  XmTextFieldSetString(host_field, "");
	}
      XmStringFree(address);
    }
  
}
		
/* Unactivate user (delete button) */
void delete_usr(Widget w, XtPointer app_data, XtPointer widget_data)
{
  int sel_no;
  XmStringTable sel_item;

  XtVaGetValues(activeusr_list,
		XmNselectedItemCount, &sel_no,
		NULL);
  if(sel_no > 0)
    {
      XtVaGetValues(activeusr_list,
		    XmNselectedItems, &sel_item,
		    NULL);
      XmListDeleteItem(activeusr_list, sel_item[0]);
    }  
}

/* Invite active users */
void invite_usrs(Widget w, XtPointer app_data, XtPointer widget_data)
{
  int item_no;
  XmStringTable items;
  char *item;
  int i;
  Participant part;

  XtVaGetValues(activeusr_list,
		XmNitemCount, &item_no,
		NULL);
  if(item_no > 0)
    {
      XtVaGetValues(activeusr_list,
		    XmNitems, &items,
		    NULL);
      for(i=0; i<item_no; i++)
	{
	  XmStringGetLtoR(items[i], XmSTRING_DEFAULT_CHARSET, &item);
	  parseIPaddress(item, part.user, part.host);
	  XtFree(item);
	  ncsSessionAddMember(&part);
	}  
    }
  XtDestroyWidget(mainWin);
}


/********************* Add members dialog END ***************************/


/************************************************************************
 *
 * Callbacks regarding plugins management 
 * 
 ************************************************************************/
void info_plugin(Widget w, XtPointer data1, XtPointer data2)
{
}

void show_plugin(Widget w, XtPointer data1, XtPointer data2)
{
}

void hide_plugin(Widget w, XtPointer data1, XtPointer data2)
{
}

void accept_invitation(Widget w, XtPointer data1, XtPointer data2)
{
  ncsSessionAcceptInvitation(invitingPart);
  SetMessage("Invitation to teleconference has been accepted!");
  XtSetSensitive(accept_but, False);
  XtSetSensitive(reject_but, False);
    
  return;
}

void reject_invitation(Widget w, XtPointer data1, XtPointer data2)
{
  SetMessage("Invitation to teleconference has been rejected!");
  XtSetSensitive(accept_but, False);
  XtSetSensitive(reject_but, False);
  return;
}


/***************************************************************************
 * 
 * DVE Events callbacks
 * 
 ***************************************************************************/
void partyRejected(Participant *part)
{
  char text[100];

  sprintf(text, "User %s@%s\n rejected invitation!\n", 
	  part->user, 
	  part->host);
  showMessage(parent, text);
}

void partyLeft(Participant *part)
{
  char text[100];
  XmString str;

  sprintf(text, "%s@%s", part->user, part->host);
  str = XmStringCreateSimple(text);
  if(XmListItemExists(membersAccList, str))
    XmListDeleteItem(membersAccList, str);
  XmStringFree(str);
}

void partyNew(Participant *part)
{
  char text[100];
  XmString str;

  sprintf(text, "%s@%s", part->user, part->host);
  str = XmStringCreateSimple(text);
  XmListAddItem(membersAccList, str, 0);
  XmStringFree(str);
}

void invited(Participant *part)
{
  char text[100];
  int res;

  sprintf(text, "RECEIVED INVITATION!\n from: %s@%s\n", 
	  part->user, 
	  part->host);
  showMessage(parent, text);

  XtSetSensitive(accept_but, True);
  XtSetSensitive(reject_but, True);
  SetMessage("Press ACCEPT or REJECT");

  memcpy(&invitingPart, part, sizeof(Participant));
  
  return;
}

static void
SetMessage(const char *msg)
{
  XmString str=XmStringCreateSimple((char*)msg);
  XtVaSetValues(statuslabel,
		XmNlabelString, str,
		NULL);
  XmStringFree(str);
  return;
}

/***************************************************************************

  Definitions of OpenDVE-specific functions

***************************************************************************/  

void start_conference(void)
{ 
  int err;
 
  SetMessage("Initializing Conference Engine...");
  if ((err = check_process("/home/R11A/stach/CE/Daemon/engine", "")) < 0)
    {
      SetMessage("Can not initialize NCS... Aborted...");
      return;
    }

  ncsSessionInitialize();
  ncsSessionCreate();

  ncsSessionAddCallback(indLocalSessionInvited, invited);
  ncsSessionAddCallback(indLocalSessionAddMember, partyNew);
  ncsSessionAddCallback(indLocalSessionDeleteMember, partyLeft);

  /* NCS start working procedure reading messages */
  recworkproc = XtAppAddWorkProc(AppContext, recWorkProc, NULL);

  SetMessage("Connected to Conference Engine...");
  fflush(stdout);
  SetMessage("Initializing conferencing panel...");

  if ((err = init_plugins())<0)
    { 
      SetMessage("Can not initialize conferencing panel(plugins)... Aborting...");
    }
  
  createMembersInterface();
  
  SetMessage("Conferencing panel ready to use.");
}


void exit_conference(void)
{
  XtRemoveWorkProc(recworkproc);
  ncsSessionQuit();
}


/**********************************************************************
 *
 * Plugins list procedures
 *
 *********************************************************************/
void pluginSelected(Widget w, XtPointer data1, XtPointer data2)
{
  XmListCallbackStruct *cbs=(XmListCallbackStruct *)data2;

/*  printf("Selected item: %d\n", cbs->item_position-1);
  printf("Number of items: %d\n", cbs->selected_item_count);
  fflush(stdout);*/
  if (cbs->selected_item_count > 0)
  {
    XtSetSensitive(show_but, True);
    XtSetSensitive(hide_but, True);
    XtSetSensitive(info_but, True);
  }
  else
  {
    XtSetSensitive(show_but, False);
    XtSetSensitive(hide_but, False);
    XtSetSensitive(info_but, False);
  }
  
  return;
}

void createPluginsInterface()
{
  int i;
  XmString label;

  label=XmStringCreateLocalized("Available plugins:");
  pluginsLabel=XtVaCreateManagedWidget("pluginsLabel",
				       xmLabelGadgetClass,
				       listform,
				       XmNlabelString, label,
				       XmNx, 10,
				       XmNy, 10,
				       NULL);


  numArg=0;
  XtSetArg(wArg[numArg], XmNx, 10); numArg++;
  XtSetArg(wArg[numArg], XmNy, 30); numArg++;
  XtSetArg(wArg[numArg], XmNwidth, 220); numArg++;
  XtSetArg(wArg[numArg], XmNlistSizePolicy, XmCONSTANT); numArg++;
  pluginsList = XmCreateScrolledList(listform, "Plugins", wArg, numArg);

  plugTlist=NULL;
  XtVaSetValues(pluginsList,
	        XmNitems, plugTlist,
		XmNitemCount, 0,
		XmNvisibleItemCount, 6,
		XmNselectionPolicy, XmSINGLE_SELECT,
		NULL);
					   		
  /*
  plugTlist=(XmStringTable)XtMalloc(appData.numPlug*sizeof(XmString *));
  for(i=0; i<appData.numPlug;i++)
  {
    plugTlist[i]=XmStringCreateLocalized(dvePluginList[i]->title);
  }
  */
  
  XtAddCallback(pluginsList, XmNsingleSelectionCallback, pluginSelected, NULL);
  XtManageChild(pluginsList);
  
  /*
  for(i=0; i<appData.numPlug;i++)
  {
    XmStringFree(plugTlist[i]);
  }
  
  XtFree(plugTlist);
  */
  return;
}

int init_plugins()
{
  char *home;
  
  /*
  appData.initPath=(char*)XtMalloc(256);  
  if (appData.initPath == NULL)
  {
     printf("Can not allocate memory for string! Exiting...\n");
     return -1;
   }

  home = getenv("HOME");
  strcpy(appData.initPath, home);
  strcat(appData.initPath, "/plugins.cfg");
  
  err = dvePluginMgrInitialize(session, appData.initPath, PluginReady, PluginDone);

  if (err < 0)
    {
      fprintf(stderr, "Error %d on dvePluginMgrInitialize().\n", 
	      dvePluginMgrErrno);
      return -1;
    }
  XtFree(appData.initPath);
  
  appData.numPlug = dvePluginMgrIdentifyAvailablePlugins(session);
  */

  createPluginsInterface();
  return 0;
}


/************************************************************************
 *
 * Members list procedures
 *
 ************************************************************************/
void createMembersInterface()
{
  int i;

  /* list of members which accepted invitation */
  label=XmStringCreateLocalized("Active members:");
  membersLabel=XtVaCreateManagedWidget("membersLabel",
				       xmLabelGadgetClass,
				       listform,
				       XmNlabelString, label,
				       XmNx, 240,
				       XmNy, 10,
				       NULL);

  
  numArg=0;
  XtSetArg(wArg[numArg], XmNx, 240); numArg++;
  XtSetArg(wArg[numArg], XmNy, 30); numArg++;
  XtSetArg(wArg[numArg], XmNwidth, 250); numArg++;
  XtSetArg(wArg[numArg], XmNlistSizePolicy, XmCONSTANT); numArg++;
  membersAccList = XmCreateScrolledList(listform, "membersAcc",
					       wArg, numArg);

  XtVaSetValues(membersAccList,
		XmNitemCount, 0/*appData.numInv*/,
		XmNvisibleItemCount, 6,
		XmNselectionPolicy, XmMULTIPLE_SELECT,
		NULL);
					   		
  XtManageChild(membersAccList);
  
}



/*
 * function adding members from http file to adding list
 */
void parseInformation(Widget list, char *info)
{
  int pos = 0;
  char user[100];
  XmString xmstring;
  int number, i;

  sscanf(info, "%d\n", &number);
  sscanf(info, "%s\n", user);
  pos = strlen(user) + 1;
  for(i=0; i<number; i++)
    {
      sscanf(&info[pos], "%s\n", user);
      pos += strlen(user) + 1;
      xmstring = XmStringCreateSimple(user);
      XmListAddItem(list, xmstring, 0);
      XmStringFree(xmstring);
    }

}

Boolean recWorkProc()
{
  struct timeval timeout;
  fd_set set;
  int sockfd;

  timeout.tv_sec = 0;
  timeout.tv_usec = 100000;
  sockfd = ncsSessionGetFD(&set);
  if(select(sockfd+1, &set, 0, 0, &timeout) > 0)
    {
      ncsProcessIncoming(&set);
    }

  return False;
}










