/*********************************************************************
 *
 * File:         control_clbks.c
 *
 * Description:  Messages callbacks
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-14-96
 *
 *********************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>

#include <netdb.h>
#include <unistd.h>

#include "global.h"


/**************************************************************************
 *
 * Callbacks for local control messages
 *
 **************************************************************************/
void localClbSessionAddMember(Message *msg, int len)
{
  Participant *part;
  InviteData *data;
  int no;
  int size;

  part = (Participant *)msg->data;
  
  printf("Engine: invitation send to %s@%s\n", part->user, part->host);
  no = sessionGetNoMembers();
  if(no > 0)
    size = (no - 1) * sizeof(Participant) + sizeof(InviteData);
  else
    size = sizeof(InviteData);
  data = malloc(size);
  data->numMembers = no;
  sessionGetMembers(data->parts);
  sendControlMessageToUser(part, reqRemoteSessionInvite, 
			   data, size);
  free(data);
}

void localClbSessionJoin(Message *msg, int len)
{
  Participant *part;

  part = (Participant *)msg->data;
  sessionActivate();
  sendControlMessageToUser(part, reqRemoteSessionAccept, NULL, 0);  
}

void localClbSessionGetMembers(Message *msg, int len)
{
  InviteData *data;
  int no, size;

  no = sessionGetNoMembers();
  if(no > 0)
    size = (no - 1) * sizeof(Participant) + sizeof(InviteData);
  else
    size = sizeof(InviteData);
  data = malloc(size);
  data->numMembers = no;
  sessionGetMembers(data->parts);
  sendControlMessage(LOCAL, -1, indLocalSessionMembers, 
			   data, size);
  free(data);
}

void localClbSessionQuit(Message *msg, int len)
{
  sendControlMessage(REMOTE, -1, reqRemoteSessionQuit, NULL, 0);
}

void localClbAppAttach(Message *msg, int len)
{
  sessionAddApp((struct sockaddr_in *)msg->data);
}

void localClbAppDettach(Message *msg, int len)
{
  sessionDeleteApp(*(int *)msg->data);
}

/**************************************************************************
 *
 * Callbacks for remote control messages
 *
 **************************************************************************/
void remoteClbSessionInvite(Message *msg, int len)
{
  InviteData *data;
  int no;
  int i;
  int size;

  size = len - sizeof(Message) + 1;
  data = (InviteData *)msg->data;
  if(sessionCreate(&data->parts[0]))
    {
      sessionDesactivate();
      sendControlMessage(LOCAL, 0, indLocalSessionInvited, 
			 &msg->sender, sizeof(Participant));  
    }
  else
    sendControlMessageToUser(&msg->sender, reqRemoteSessionReject, 
			     NULL, 0);
}


void remoteClbSessionAddRest(Message *msg, int len)
{
  InviteData *data;
  int i, no, size;

  size = len - sizeof(Message) + 1;
  data = (InviteData *)msg->data;
  no = data->numMembers;
  for(i=1; i<no; i++)
    sessionAddMember(&data->parts[i]);

  sessionAddMember(&msg->receiver);
  sessionSetLocalMember();
}

void remoteClbSessionAccept(Message *msg, int len)
{
  InviteData *data;
  int size, data_size, no;

  size = len - sizeof(Message) + 1;

  no = sessionGetNoMembers();
  if(no > 0)
    data_size = (no - 1) * sizeof(Participant) + sizeof(InviteData);
  else
    data_size = sizeof(InviteData);
  data = malloc(data_size);
  data->numMembers = no;
  sessionGetMembers(data->parts);
  sendControlMessageToUser(&msg->sender, reqRemoteSessionAddRest,
			   data, data_size);
  free(data);

  sendControlMessage(REMOTE, -1, reqRemoteSessionAddMember, 
		     &msg->sender, sizeof(Participant));

  sessionAddMember(&msg->sender);

  sendControlMessage(LOCAL, 0, indLocalSessionAccept, 
		     msg->data, size);
}

void remoteClbSessionReject(Message *msg, int len)
{
  int size;

  size = len - sizeof(Message) + 1;
  sendControlMessage(LOCAL, 0, indLocalSessionReject, 
		     msg->data, size);  
}


void remoteClbSessionAddMember(Message *msg, int len)
{
  int size;

  size = len - sizeof(Message) + 1;
  sessionAddMember((Participant *)msg->data);
}

void remoteClbSessionQuit(Message *msg, int len)
{
  sessionDeleteMember(&msg->sender);
}  








