/*********************************************************************
 *
 * File:         global.h
 *
 * Description:  Global definitions for conference engine
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-14-96
 *
 *********************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>

#include <netdb.h>
#include <unistd.h>

#define MAX_UDP_SIZE 60000

#define MAX_MEMBERS 20
#define MAX_APPS 20

#define DAEMON_PORT 5000
#define MANAGER_PORT 4999

typedef struct _participant
{
  char user[20];
  char host[50];
} Participant;

typedef struct _message
{
  unsigned char local;
  unsigned short tag;
  Participant sender;
  Participant receiver;
  char data[1];
} Message;

/* Tags in local messages */
typedef enum _local_tag
{
  /* local requests */
  reqLocalSessionCreate,          /* 0 */
  reqLocalSessionQuit,
  reqLocalSessionInit,
  reqLocalSessionAddMember,
  reqLocalSessionJoin,
  reqLocalSessionGetMembers,      /* 5 */

  reqLocalApplicationAttach,
  reqLocalApplicationDettach,

  /* local indications */
  indLocalSessionInvited,
  indLocalSessionAccept,
  indLocalSessionReject,          /* 10 */
  indLocalSessionMembers,
  indLocalSessionAddMember,
  indLocalSessionDeleteMember,
  indLocalSessionLocalMember
} LocalTag;

typedef enum _remote_tag
{
  /* remote requests */
  reqRemoteSessionInvite,
  reqRemoteSessionAccept,
  reqRemoteSessionQuit,
  reqRemoteSessionReject,
  reqRemoteSessionAddMember,
  reqRemoteSessionAddRest,

  /* remote acknowledgments */
  ackRemoteInvitationReceived
} RemoteTag;

typedef struct _invite_data
{
  int numMembers;
  Participant parts[1];
} InviteData;


typedef struct _member_info
{
  int handle;
  Participant part;
  struct sockaddr_in addr;
} MemberInfo;

#define LOCAL 1
#define REMOTE 0

int sendControlPacket(struct sockaddr_in *addr, char *msg, int len);
int sendControlMessage(char local, int user, short tag, char *data, int len);
int sendControlMessageToUser(Participant *part, short tag, 
			     char *data, int len);

