#include <stdio.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xlib.h>
#include <dve/Xm/recipientList.h>

#include <Xm/Xm.h>

#ifdef MOTIF
#include <Xm/Xm.h>
#include <Xm/MwmUtil.h>
#include <Xm/MenuShell.h>
#include <Xm/RowColumn.h>
#include <Xm/Scale.h>
#include <Xm/MainW.h>
#endif /* MOTIF */

#include <Xm/PushB.h>
#include <Xm/Form.h>
#include <Xm/ToggleB.h>
#include <Xm/Label.h>
#include <X11/Shell.h>

XtAppContext	AppContext;
Widget		TopLevel;
Display		*display;
int		screen;
Widget MyShell;
Widget MainForm;
Widget bPlay, bFreeze, bExit;
Widget messagelabel;
Widget option_menu;
XmString label, str1, str2;

extern finish();
extern start_playing();
extern void play();
extern void freeze();
void SetMessage(const char *msg);
void option_cb();
char *localName;

int cos=0;

Widget build_interface(Widget parent);

/*----------------------------------------------
 * Insert application global declarations here
 *---------------------------------------------*/


#ifdef _NO_PROTO
main(argc,argv)
        int     argc;
        char    *argv[];
#else
main( int argc, char *argv[])
#endif /* _NO_PROTO */
{
	/*-----------------------------------------------------------
	 * Declarations.
         * The default identifier - mainIface will only be declared 
	 * if the interface function is global and of type swidget.
	 * To change the identifier to a different name, modify the
	 * string mainIface in the file "xtmain.dat". If "mainIface"
         * is declared, it will be used below where the return value
	 * of  PJ_INTERFACE_FUNCTION_CALL will be assigned to it.

         *----------------------------------------------------------*/ 

  	Widget mainIface;

	/*---------------------------------
	 * Interface function declaration
	 *--------------------------------*/	

	/*---------------------
	 * Initialize program
	 *--------------------*/

  	TopLevel = XtAppInitialize(&AppContext, "VideoPluginMotif",
				     NULL, 0, &argc, argv, NULL, NULL, 0);
	display = XtDisplay(TopLevel);
	screen = XDefaultScreen(display);

	/* We set the geometry of UxTopLevel so that dialogShells
	   that are parented on it will get centered on the screen
	   (if defaultPosition is true). */

	XtVaSetValues(TopLevel,
			XtNx, 0,
			XtNy, 0,
			XtNwidth, DisplayWidth(display, screen),
			XtNheight, DisplayHeight(display, screen),
			NULL);

	/*-------------------------------------------------------
	 * Insert initialization code for your application here 
	 *------------------------------------------------------*/
	
	init_codec();
	init_encoder();
	init_decoder();

	/*----------------------------------------------------------------
	 * Create and popup the first window of the interface.  The 	 
	 * return value can be used in the popdown or destroy functions.
         * The Widget return value of  PJ_INTERFACE_FUNCTION_CALL will 
         * be assigned to "mainIface" from  PJ_INTERFACE_RETVAL_TYPE. 
	 *---------------------------------------------------------------*/

	introduceYourself();

	mainIface = build_interface(TopLevel);

	XtPopup(MyShell, XtGrabNone);

	/*-----------------------
	 * Enter the event loop 
	 *----------------------*/

  	XtAppMainLoop (AppContext);

}


Widget build_interface(Widget parent)
{
  int xval, yval;
  cos=176;

#ifdef MULTI
  xval=250;
  yval=100;
#else
  xval=300+cos;
  yval=240+cos;
#endif
  
    MyShell = XtVaCreatePopupShell( "recipTestAppShell",
				 applicationShellWidgetClass,
				 parent,
				 XmNx, 370,
				 XmNy, 300,
				 XmNwidth, 300,
				 XmNheight, 150,
				 XmNtitle, localName,
				 XmNiconName, "VideoPlugin",
				 XmNmwmDecorations, 2|16|32,
				 XmNmwmFunctions, 4|8|32,
				 NULL );

	MainForm = XtVaCreateManagedWidget( "recipTestMainForm",
			xmFormWidgetClass,
			MyShell,
			XmNresizePolicy, XmRESIZE_NONE,
			XmNunitType, XmPIXELS,
			XmNwidth, xval,
			XmNheight, yval,
			NULL );


	label = XmStringCreateSimple("PLAY");
	bPlay = XtVaCreateManagedWidget("play", xmPushButtonWidgetClass,
					 MainForm,
					 XmNlabelString, label, 
					 XmNx, 10, 
					 XmNy, 50,
					 XmNwidth, 65,
					 XmNheight, 30,
					 NULL);

	XmStringFree(label);
	XtAddCallback(bPlay, XmNactivateCallback, play, NULL);
	XtSetSensitive(bPlay, True);


	label = XmStringCreateSimple("Freeze");
	bFreeze = XtVaCreateManagedWidget("freeze", xmPushButtonWidgetClass,
					 MainForm,
					 XmNlabelString, label, 
					 XmNx, 80, 
					 XmNy, 50,
					 XmNwidth, 85,
					 XmNheight, 30,
					 NULL);

	XmStringFree(label);
	XtAddCallback(bFreeze, XmNactivateCallback, freeze, NULL);
	XtSetSensitive(bFreeze, False);

	label = XmStringCreateSimple("Exit");
	bExit = XtVaCreateManagedWidget("Exit", xmPushButtonWidgetClass,
					 MainForm,
					 XmNlabelString, label, 
					 XmNx, 170, 
					 XmNy, 50,
					 XmNwidth, 65,
					 XmNheight, 30,
					 NULL);

	XmStringFree(label);
	XtAddCallback(bExit, XmNactivateCallback, finish, NULL);
	XtSetSensitive(bExit, True);

  label = XmStringCreateSimple("Compression:");
  str1 = XmStringCreateSimple("H263");
  str2 = XmStringCreateSimple("H261");
  option_menu = XmVaCreateSimpleOptionMenu(MainForm, "option_menu", 
		  label, 'C', 0, option_cb,
		  XmVaPUSHBUTTON, str1, '3', NULL, NULL,
		  XmVaPUSHBUTTON, str2, '1', NULL, NULL,
					   XmNx, 10, 
					   XmNy, 80,
					   XmNwidth, 65,
					   XmNheight, 30,
		  NULL);
  XmStringFree(str1);
  XmStringFree(str2);
  XmStringFree(label);

  XtManageChild(option_menu);

	messagelabel = XtVaCreateManagedWidget( "messagelabel",
			xmLabelWidgetClass,
			MainForm,
			XmNx, 20,
			XmNy, 10,
			XmNwidth, 300,
			XmNheight, 10,
			NULL );
	SetMessage("Press PLAY to start...");

  return MyShell;
}

void SetMessage(const char *msg)
{
  XmString str=XmStringCreateSimple((char*)msg);
  XtVaSetValues(messagelabel,
		XmNlabelString, str,
		NULL);
  XmStringFree(str);
}


