/***********************************************************************
 *
 * Name:        Plugin_procs.c
 *
 * Description: user and OpenDVE events callbacks               
 *
 * Author:	Janusz Bulawa & Tomasz Stachowiak, NPAC
 *
 * Date:	September 1996
 *
 ***********************************************************************/
 
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <sys/param.h>
#include "VideoProtocol.h"

#include <X11/Intrinsic.h>

#include <Xm/Xm.h>
#include <Xm/MwmUtil.h>
#include <Xm/MenuShell.h>
#include <Xm/RowColumn.h>
#include <Xm/Scale.h>
#include <Xm/MainW.h>

#include "../lib/common.h"

/* H263 display section */
#define GLOBAL
#include "config.h"
#include "tmndec.h"
#include "global.h"

void initdecoder _ANSI_ARGS_((void));

/* private prototypes */
static int getval _ANSI_ARGS_((char *argv[]));

int first[MAX_USERS];

/* end of H263 display section */

extern app;
extern toplevel;
extern start_but;
extern stop_but;
extern shell;

extern localName;
int localHandle;
int    currentUser;
extern unsigned char *frame;
extern char* videobuffer;
extern int numbytes;
extern int writeoffset;
extern int expand;

Window window;
Window win[MAX_USERS];
int dpy_depth;
int depth[MAX_USERS];

extern Display *display;

int totalBytesSent=0;
int totalFramesSent=0;

static unsigned char *localPict[3];
static unsigned char *exlocalPict[3];

Widget MainWindow;

ncsApplication *ncsapp;
XtWorkProc recworkproc;

Boolean recWorkProc();

extern state;
int tmp_state;

int canSend = 0;

/********************************
 * TIME INTERVAL BETWEEN FRAMES
 ********************************/

int interval = 100;

/********************************/

int numUsers, i;
char *wlabel;

int code_type = H263;
int busy = 0;

void start_playing();
void start_conference(void);
void finish(void);
void started();
void HidePlugin();
void ShowPlugin();
void QuitPlugin();
void play(void);
void stop(void);

void SendVideoData(char *data, int len);
/*
void SendMessage(VideoTag messageType, DvePort port, VideoPluginArg *dane);
int SendPluginMessageGeneric(DveSession dveSession, u_int32 protocol, DvePort port,
			u_int32 messageType, int dest, void *data, u_int32 size);
			*/
extern SetMessage(const char *msg);


Boolean scheduleFrame();
Boolean nextFrame();

XtIntervalId timerId=0;
extern XtWorkProc workProc;

char *data;


/*************************************************
 *
 * Data sending procedures
 *
 *************************************************/

/* Video data sending */
void SendVideoData(char *data, int len)
{
  VideoPluginFrame *frame;

   frame = (VideoPluginFrame *)malloc(sizeof(VideoPluginFrame)+len-1);
   if (!frame)
   {
     printf("Message allocation error\n");
     return;
   }

  totalBytesSent+=len;
  totalFramesSent++;
  if (len>0)
  {
    gettimeofday(&frame->time);
    memcpy(frame->data, data, len);

    printf("Sending video data\n");
    ncsAppSendData(ncsapp, -1, code_type,frame,sizeof(VideoPluginFrame)+len-1);

  }
  else printf("SendVideoData(): Illegal data length value!\n");

  return;
}

/* Special data sending */
void SendSpecificMessage(VideoTag messageType)
{
   static VideoPluginArg dane;
   char *title;

   switch (messageType) {
   case videoTagPlay:
     dane.frate=10;
     dane.format=SF_QCIF;
     break;
   case videoTagConfirm:
     break;
   case videoTagStop:
     break;
   case videoTagExit:
     break;
   case videoTagFrameDecoded:
     break;
   default:
     printf("Unknown request in SendSpecific...!\n");
     break;
   }

   ncsAppSendData(ncsapp, -1, messageType, &dane, sizeof(dane)); 
}

/* Special data sending */
void SendSpecificMessageToMember(int userNo, VideoTag messageType)
{
    ncsAppSendData(ncsapp, userNo, messageType, NULL, 0);
}



/******************************************
 *
 * Received video data processing
 *
 ******************************************/
static void
ProcessVideoData(void *data, int size, int type, int userNo)
{
  int i, f;
  VideoPluginFrame *frame;

  frame = (VideoPluginFrame *)data;
  /*doReceiveStatistics(userNo, &frame->time);*/

  memset(invideobuffer, 0, MAX_FRAME_SIZE);
  avail=size - sizeof(VideoPluginFrame) + 1;
  memcpy(invideobuffer, frame->data, avail);
  invideooffset=0;
  initbits();
  if(type == 0)
    {
       if (getheader())
	 {
	    getpicture(&framenum, 0);
	    framenum++;
	 }
    }
  else
    {
       while(busy) {};
       getpicture(&framenum, 1);
    }

  SendSpecificMessageToMember(userNo, videoTagFrameDecoded);
}


/********************************************************
 *
 * Main coding and sending procedure, invoked with timer 
 *
 ********************************************************/
Boolean nextFrame()
{
  int wait_time;
  struct timespec wait;

      if((state == PLAY))
	{
	  
	  /* frame capturing and compression */
	  switch(code_type)
	    {
	    case H263:
	      numbytes=codeFrame(&frame);
	      break;
	      
	    case H261:
	      frame = getFrame(176, 144);
	      numbytes = p64EncodeFrame(frame, videobuffer);
	      break;
	    }
	  
	  doSendStatistics(numbytes);

	  /* mirror image handling */
#ifdef LOCAL
	  localPict[0]=frame; 
	  localPict[1]=frame+25344;
	  localPict[2]=frame+25344+6336;
	  currentUser=(int)localHandle;
	  window=win[currentUser];
	  dpy_depth=depth[currentUser];
	  if (expand && outtype == T_X11) 
	    {
	      interpolate_image(localPict[0], exlocalPict[0],
				coded_picture_width, coded_picture_height);
	      interpolate_image(localPict[1], exlocalPict[1], 
				chrom_width, chrom_height);
	      interpolate_image(localPict[2], exlocalPict[2], 
				chrom_width, chrom_height);
	      dither(exlocalPict);
	    }
	  else
	    {
	      dither(localPict);
	    }
#endif
      
	  free(frame);
	  
	  /* compressed frame sending */
	  SendVideoData(videobuffer, numbytes);
	  
	  canSend = 0;

	  /*	  timerId = XtAppAddTimeOut(app, 1500, nextFrame, NULL);*/
	}

      return False;
}

void remoteFrameDecoded(int userNo)
{
  static int usersConfirmed = 0;

  usersConfirmed++;
  printf("Remote user confirmed\n");
  if(usersConfirmed == numUsers - 1)
    {
      printf("All remote users confirmed\n");
      canSend = 1;
      usersConfirmed = 0;
      if(timerId != NULL)
	{
	  XtRemoveTimeOut(timerId);
	  timerId = NULL;
	}
      if(state == PLAY)
	nextFrame();
    }
}


/**********************************
 *
 * Main panel callbacks
 *
 **********************************/

/*
 * Buttons pushing callbacks
 */
void start_pushed(Widget w, XtPointer data, XmPushButtonCallbackStruct *cbs)
{
  XtSetSensitive(start_but, False);	 
  XtSetSensitive(stop_but, True);  

  state=PLAY;
  SendSpecificMessage(videoTagPlay);

  
  source_format=SF_QCIF;

  sendStart();
  if (first[localHandle]) 
    {
       initdpy(localHandle);
       first[localHandle] = 0; 
    }
   
    
  start_playing();
  SetMessage("Video is running...");
  canSend = 1;
  
  return;
}

void stop_pushed(Widget w, XtPointer data, XmPushButtonCallbackStruct *cbs)
{
  XtSetSensitive(start_but, True);
  XtSetSensitive(stop_but, False);

  state=STOP;
  sendStop();
  SendSpecificMessage(videoTagStop);

  SetMessage("Video stopped...");
}

/*
 * File menu callback
 */
void file_menu(Widget menu_item, int item_no,
		  XmAnyCallbackStruct *cbs)
{
   switch(item_no)
     {
	case 0:
	SendSpecificMessage(videoTagExit);
	QuitPlugin();
	break;
     }
}


/*
 * Tool menu callback 
 */
void tool_menu(Widget menu_item, int item_no,
		  XmAnyCallbackStruct *cbs)
{
   switch(item_no)
     {
	case 0:
	  if (timerId!=0) 
	    {
	      XtRemoveTimeOut(timerId);
	      timerId=0;
	    }
	  tmp_state = state;
	  if(state == PLAY)
	    {
	      SendSpecificMessage(videoTagStop);
	      sendStop();
	    }
	  state = PANEL;
	  if(shell==NULL) PanelDialog();
	  else XtPopup(shell, XtGrabNone);
	  break;

        case 1:
	  sendStop();
	  sendTimeReport();
	  sendStart();
	  break;
     }
}


/****************************************************
 *
 * Video control panel callbacks
 *
 ***************************************************/

/* Compression type menu callback */
void
comp_option_cb(w, item_no, cbs)
Widget w;
int item_no;
XmAnyCallbackStruct *cbs;
{
   switch(item_no)
     {
	case 0:
	code_type = H263;
	break;

	case 1:
	code_type = H261;
	break;
     }
}

/* Video size menu callback */
void
size_option_cb(w, item_no, cbs)
Widget w;
int item_no;
XmAnyCallbackStruct *cbs;
{
   switch(item_no)
     {
	case 0:
	printf("QCIF size\n");;
	break;

	case 1:
	printf("CIF size\n");;
	break;
     }
}

/* expand toggle button callback */
void expand_cb(Widget w, char *data, XmToggleButtonCallbackStruct *cbs)
{
  int err, i;


  if(cbs->set)
    {
      expand = 1;
      for(i=0; i<MAX_USERS; i++)
	{
	  if(win[i] != NULL)
	    {
	      err = XDestroyWindow(display, win[i]);
	      initdpy(i);
	    }
	}
    }
	       
   else
     {
       expand = 0;
       for(i=0; i<MAX_USERS; i++)
	 {
	   if(win[i] != 0)
	     {
	       XDestroyWindow(display, win[i]);
	       initdpy(i);
	     }
	 }
     }
}

/* intra mode toggle button callback */
void intra_cb(Widget w, char *data, XmToggleButtonCallbackStruct *cbs)
{
   if(cbs->set)
     setIntraMode();
   else
     resetIntraMode();

}

/*
 * Video panel dialog callbacks
 */
void v_file_menu(Widget menu_item, int item_no,
		  XmAnyCallbackStruct *cbs)
{
   switch(item_no)
     {
	case 0:
	  state = tmp_state;
	  if(state == PLAY)
	      SendSpecificMessage(videoTagPlay);
	  XtPopdown(shell);
	  if(state == PLAY)
	    {
	      start_playing();
	      sendStart();
	    }
	  break;
     }
}




void start_playing()
{
  state=PLAY;

  initcapture(176, 144);

  if(code_type == H263)
    {
      frame = getFrame(176, 144);
      codeIntraFrame(frame);
      writeoffset=0;
      SendVideoData(videobuffer, numbytes);
    }
  else
    {
      canSend = 1;
      nextFrame();
    }
}

void starting()
{
  localName=(char*)malloc(64);
  localHandle=ncsAppGetLocalHandle(ncsapp);
  printf("My handle %d  !!!\n", localHandle); 
  numUsers = ncsAppGetNumMembers(ncsapp);
}


/****************************************
 *
 * DVE functions
 *
 ***************************************/
disconnectPlugin()
{
  ncsAppDettach(ncsapp);
  exit(0);
}

void VideoControlMsg(int sender, short type, void *data, int len)
{
  static VideoPluginArg dane;

  memcpy(&dane, data, sizeof(VideoPluginArg));

  switch (type)
   { 
      case H263:
	 if ((sender>numUsers))
	 {
	   printf("Invalid sender handle: %d\n", sender);
	   sender=1;
	 }
	 currentUser = (int)sender;
	 window = win[currentUser];
	 dpy_depth = depth[currentUser];
         ProcessVideoData(data, len, 0, sender);
	 break;

      case H261:
	 if ((sender>numUsers))
	 {
	   printf("Invalid sender handle: %d\n", sender);
	   sender=1;
	 }
	 currentUser = (int)sender;
	 window = win[currentUser];
	 dpy_depth = depth[currentUser];
         ProcessVideoData(data, len, 1, sender);
	 break;

      case videoTagExit:
         if (timerId!=0) XtRemoveTimeOut(timerId);
	 timerId=0;
	 QuitPlugin();
         break;

      case videoTagConfirm:
	break;

      case videoTagPlay:
	 source_format=dane.format;
         if (first[sender]) {
           initdpy(sender);
	   usrTimeInit(sender);
           first[sender] = 0;
         }
	 receiveStart(sender);
         SendSpecificMessage(videoTagConfirm);
	 SetMessage("Receiving video...");
         break;

       case videoTagStop:
	 receiveStop(sender);
         break;

       case videoTagFrameDecoded:
	 remoteFrameDecoded(sender);
	 break;

       default:
	 printf("Unknown request in VideoControlMsg %d from %d!\n", type, sender);
	 break;
   }
    
}

void
HidePlugin()
{
   XtPopdown(toplevel);
}
 
void	
ShowPlugin()
{
   XtPopup(toplevel, XtGrabNone);
}
 
void
QuitPlugin()
{
  /*  printf("Video Plugin: EXIT requested. Exiting...\n");
  printf("Average video data rate: %5.2f bytes per frame\n",
	 (float)totalBytesSent/(float)totalFramesSent);*/
  if(state == PLAY)
    sendStop();
  fflush(stdout);
  disconnectPlugin();
  shutdownEncoder();
  exit(0);
}
 
void
introduceYourself()
{
   /*
    * Start up NCS.
    */
  ncsapp = ncsAppAttach(VIDEO_PLUGIN_PORT);
  recworkproc = XtAppAddWorkProc(app, recWorkProc, NULL);

  ncsAppAddCallback(ncsapp, tagAppStarting, starting);

  /* Uzupelnic localName, localHandle i numUsers */
   /*
   localName=dveGetLocalHostName(dvePluginSession);
   numUsers=dveGetAllMembers(dvePluginSession, allUsers, MAX_USERS);
   */

   if (numUsers>MAX_USERS)
   {
     printf("Too many users for videokonference! Exiting...\n");
     fflush(stdout);
     exit(-1);
   }
     
   
  for(i=0; i<MAX_USERS; i++)
    first[i]=1;

  quiet=1; 
  ld = &base; 

  framenum = 0;
  temp_ref = 0;
  prev_temp_ref= -1;
  state=STOP; 
  outputname = "";
  videobuffer=(char*)malloc(MAX_FRAME_SIZE);
  invideobuffer=(char*)malloc(MAX_FRAME_SIZE);
  vdata = malloc(MAX_FRAME_SIZE);
  avail=0;

   return;
}

/*
 * Notify other conference members of our current settings.  We have
 * either made a change or have received a request to provide status.
 */
/*
void
SendMessage(VideoTag messageType, DvePort port, VideoPluginArg *dane)
{

   SendPluginMessageGeneric(dvePluginSession, port, port, messageType, -1, dane, sizeof(*dane));

   return;
}
*/
/*
 * Passses data from this plugin to other conferenced members.
 */
/*
int
SendPluginMessageGeneric(DveSession dveSession, u_int32 protocol, DvePort port,
			u_int32 messageType, int dest, void *data, u_int32 size)
{
   DveDataPacket *packet;
   DveConfParticipant member;
   int returnCode,i;

   packet = (DveDataPacket*) dveMallocDataPacket(dveSession, size);
   packet->tag = (u_int32)messageType;
   packet->pluginProtocol = protocol;
   packet->destPort = port;

   DvePartListClrAll(&(packet->destParts));

   if (dest < 0)
   {
      DvePartListSetAll(&(packet->destParts));
   }
   else
   {
      DvePartListSet(dest,&(packet->destParts));
   }

   if (size > 0)
      memcpy(packet->data, data, size);

   returnCode = dveSendDataPacket(dveSession, packet);

   dveFreeDataPacket(packet); 
   return(returnCode);
}
*/



/* H263 display section */
initdpy(int userno)
{
  MainWindow = XtWindow(toplevel);

  wlabel=(char*)malloc(128);
  init_display("", "Display", (int)MainWindow, userno);
  free(wlabel);
}


initexpand()
{
  int cc, k, size;

  for (cc=0; cc<3; cc++) 
    {
      if (cc==0)
	size = coded_picture_width*coded_picture_height*4;
      else
	size = chrom_width*chrom_height*4;
    
      if (!(exlocalPict[cc] = (unsigned char *)malloc(size)))
	error("malloc failed\n");

      for (k=0;k<MAX_USERS;k++) 
	{
	  if (!(exnewframe[k][cc] = (unsigned char *)malloc(size)))
	    error("malloc failed\n");
	}
    }
}

void initdecoder(void)
{
  int i, k, cc, size;
  FILE *cleared;

  source_format = SF_QCIF;

  expand = 0;
  outtype = T_X11;

  /* clip table */
  if (!(clp=(unsigned char *)malloc(1024)))
    error("malloc failed\n");

  clp += 384;

  for (i=-384; i<640; i++)
    clp[i] = (i<0) ? 0 : ((i>255) ? 255 : i);

  /* MPEG-1 = TMN parameters */
  matrix_coefficients = 5;

  switch (source_format) {
    case (SF_SQCIF):
      horizontal_size = 128;
      vertical_size = 96;
      break;
    case (SF_QCIF):
      horizontal_size = 176;
      vertical_size = 144;
      break;
    case (SF_CIF):
      horizontal_size = 352;
      vertical_size = 288;
      break;
    case (SF_4CIF):
      horizontal_size = 704;
      vertical_size = 576;
      break;
    case (SF_16CIF):
      horizontal_size = 1408;
      vertical_size = 1152;
      break;
    default:
      printf("ERROR: Illegal input format\n");
      exit(-1);
      break;
  }


  mb_width = horizontal_size/16;
  mb_height = vertical_size/16;
  coded_picture_width = horizontal_size;
  coded_picture_height = vertical_size;
  chrom_width =  coded_picture_width>>1;
  chrom_height = coded_picture_height>>1;
  blk_cnt = 6;

  for (cc=0; cc<3; cc++) {
    if (cc==0)
      size = coded_picture_width*coded_picture_height;
    else
      size = chrom_width*chrom_height;

    for (k=0;k<MAX_USERS;k++) 
      {
	if (!(refframe[k][cc] = (unsigned char *)malloc(size)))
	  error("malloc failed\n");
	if (!(oldrefframe[k][cc] = (unsigned char *)malloc(size)))
	  error("malloc failed\n");
	if (!(bframe[k][cc] = (unsigned char *)malloc(size)))
	  error("malloc failed\n");
      }
  }

  for (cc=0; cc<3; cc++) {
    if (cc==0) {
      size = (coded_picture_width+64)*(coded_picture_height+64);
      for (k=0;k<MAX_USERS;k++)
	{
	  if (!(edgeframeorig[k][cc] = (unsigned char *)malloc(size)))
	    error("malloc failed\n");
	  edgeframe[k][cc] = edgeframeorig[k][cc] + (coded_picture_width+64) * 32 + 32;
	}
    }
    else {
      size = (chrom_width+32)*(chrom_height+32);
      for (k=0;k<MAX_USERS;k++) 
	{
	  if (!(edgeframeorig[k][cc] = (unsigned char *)malloc(size)))
	    error("malloc failed\n");
	  edgeframe[k][cc] = edgeframeorig[k][cc] + (chrom_width+32) * 16 + 16;
	}
    }
  }

  /*  if (expand) 
    {*/
      initexpand();
      /*    }*/

  /* IDCT */
  if (refidct)
    init_idctref();
  else
    init_idct();

  for(i=0; i<MAX_USERS; i++)
    {
      win[i] = NULL;
    }

  init_global_dpy();
}

void error(text)
char *text;
{
  fprintf(stderr,text);
  exit(1);
}

/* trace output */
void printbits(code,bits,len)
int code,bits,len; 
{
  int i;
  for (i=0; i<len; i++)
    printf("%d",(code>>(bits-1-i))&1);
}


static int getval(argv)
char *argv[];
{
  int val;

  if (sscanf(argv[1]+2,"%d",&val)!=1)
    return 0;

  while (isdigit(argv[1][2]))
    argv[1]++;

  return val;
}


/* end of H263 display section */


Boolean recWorkProc()
{
  struct timeval timeout;
  fd_set set;
  ncsData *ncsdata;
  int size;
  int sockfd;
  char *data;

  timeout.tv_sec = 0;
  timeout.tv_usec = 100000;

  sockfd = ncsAppGetFD(ncsapp, &set);
  if(select(sockfd+1, &set, 0, 0, &timeout) > 0)
    {
      ncsAppProcessIncoming(ncsapp, &set);
      size = ncsAppGetData(ncsapp, &set, &data);
      if(size > 0)
	{
	  size = size - sizeof(ncsData) + 1;
	  ncsdata = (ncsData *)data;
	  VideoControlMsg(ncsdata->handle, ncsdata->type, ncsdata->data, size);
	  free(data);
	}

    }

  return False;
}




