/*********************************************************************
 *
 * File:         common.h
 *
 * Description:  Common definitions for session and application 
 *               management API
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-16-96
 *
 *********************************************************************/
#ifndef _NCS_H
#define _NCS_H

#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>

#include <netdb.h>
#include <unistd.h>

#define QUIET

#ifndef QUIET
#define PRINTF printf
#else
#define PRINTF nic 
#endif


#define MAX_UDP_SIZE 60000

#define MAX_MEMBERS 20

#define DAEMON_PORT 5000
#define MANAGER_PORT 4999

typedef struct _participant
{
  char user[20];
  char host[50];
} Participant;

typedef struct _message
{
  unsigned char local;
  unsigned short tag;
  Participant sender;
  Participant receiver;
  char data[1];
} Message;

/* Tags in local messages */
typedef enum _local_tag
{
  /* local requests */
  reqLocalSessionCreate,
  reqLocalSessionQuit,
  reqLocalSessionInit,
  reqLocalSessionAddMember,
  reqLocalSessionJoin,
  reqLocalSessionGetMembers,

  reqLocalApplicationAttach,
  reqLocalApplicationDettach,

  /* local indications */
  indLocalSessionInvited,
  indLocalSessionAccept,
  indLocalSessionReject,
  indLocalSessionMembers,
  indLocalSessionAddMember,
  indLocalSessionDeleteMember,
  indLocalSessionLocalMember
} LocalTag;

typedef struct _manager_data
{
  int socket;
  struct sockaddr_in addr;
  struct sockaddr_in daemon_addr;
} ManagerData;

typedef struct _invite_data
{
  int numMembers;
  Participant parts[1];
} InviteData;

typedef struct _member_info
{
  int handle;
  Participant part;
  struct sockaddr_in addr;
} MemberInfo;

typedef struct _ncs_application
{
  int port;
  int socket;
  int ctrl_socket;
  struct sockaddr_in addr;
  struct sockaddr_in ctrl_addr;
  struct sockaddr_in daemon_addr;
  MemberInfo member[MAX_MEMBERS];
  int memberActive[MAX_MEMBERS];
  int numMembers;
  int localHandle;
  void (* startingCallback)(void);
} ncsApplication;


typedef struct _ncs_data
{
  int type;
  int handle;
  char data[1];
} ncsData;

typedef enum _app_tag
{
  tagAppStarting
} AppTag;

#endif
